###
### Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
### All rights reserved.
###
### This file is part of Janos.
### 
### Janos is free software; you can redistribute it and/or modify it
### under the terms of the GNU General Public License, version 2, as
### published by the Free Software Foundation.
### 
### Janos is distributed in the hope that it will be useful, but
### WITHOUT ANY WARRANTY; without even the implied warranty of
### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
### GNU General Public License for more details, in the file
### COPYING.  If that file is not present, write to the Free
### Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
### 02111-1307, USA
### 
### The University of Utah and the Flux Group requests users to
### return to flux-dist@cs.utah.edu any improvements that they make.
###

###
### Define a package for compilation
###
### Expects 'PACKAGE_NAME' to be defined.  Expects PACKAGE_NAME to
### 'match' (in the Java sense) a directory that contians the classes for
### that package.
###
### This makefile fragment must be included once for each package;
### it adds rules for preprocessing the package and compiling
### all of its sources.
###


# Package-specific directory names
$(PACKAGE_NAME)_FULLNAME:=$(PACKAGE_PREFIX).$(PACKAGE_NAME)
$(PACKAGE_NAME)_SRC_DIR:=$(subst .,/,$(PACKAGE_NAME))
$(PACKAGE_NAME)_DEST_DIR:=$(subst .,/,$($(PACKAGE_NAME)_FULLNAME))

# Find the various files related to this package
$(PACKAGE_NAME)_SOURCES_GLOB:=$(SRC_DIR)/$($(PACKAGE_NAME)_SRC_DIR)/[A-Z]*.Java $(SRC_DIR)/$($(PACKAGE_NAME)_SRC_DIR)/$(NODEOS)_*.Java
$(PACKAGE_NAME)_SOURCES:=$(foreach glob, $($(PACKAGE_NAME)_SOURCES_GLOB), $(wildcard $(glob)))
$(PACKAGE_NAME)_BASE_SOURCES:=$(subst $(SRC_DIR)/,,$($(PACKAGE_NAME)_SOURCES))
$(PACKAGE_NAME)_CPPSOURCES:=$(addprefix $(DEST_DIR)/$(PACKAGE_PREFIX_DIR)/, $($(PACKAGE_NAME)_BASE_SOURCES:.Java=.java))
$(PACKAGE_NAME)_CPP_MACROS:=$(CPP_GLOBAL_MACROS)

# Minimum expected .class files.  (Inner classes and multiple classes
# per .java make this set smaller than the actual set of .class files
# created.)
$(PACKAGE_NAME)_MIN_CLASSES:=$(sort $($(PACKAGE_NAME)_CPPSOURCES:.java=.class))
$(PACKAGE_NAME)_CLASSES_GLOB:=$($(PACKAGE_NAME)_DEST_DIR)/*.class

# Add the sources to the global list of all sources
ALL_SOURCES+=$($(PACKAGE_NAME)_SOURCES)
ALL_CPPSOURCES+=$($(PACKAGE_NAME)_CPPSOURCES)
ALL_CLASSES_GLOB+=$($(PACKAGE_NAME)_CLASSES_GLOB)

# The set of expected preprocessed files (for a sanity check)
$(PACKAGE_NAME)_EXPECTED_CPP:=$(sort $($(PACKAGE_NAME)_CPPSOURCES))
$(PACKAGE_NAME)_ACTUAL_CPP_GLOB:=$(DEST_DIR)/$($(PACKAGE_NAME)_DEST_DIR)/*.java
$(PACKAGE_NAME)_ACTUAL_CPP:=$(sort $(wildcard $($(PACKAGE_NAME)_ACTUAL_CPP_GLOB)))

# A macro for finding all "extra" .java files in a target directory
# (for a sanity check)
#
# Note that the next assignment is not a ":=" assignment, so it is
# expanded when its *used*.  (e.g., after generation of .java files.)
$(PACKAGE_NAME)_EXTRA_CPP=$(strip $(filter-out  $($(PACKAGE_NAME)_EXPECTED_CPP),  $($(PACKAGE_NAME)_ACTUAL_CPP)))


# Post-processed source should be left around
.PRECIOUS: $($(PACKAGE_NAME)_CPPSOURCES)


## Generate a post-processed file (.java) from a .Java 
##
## Depends on the file(s) that defines the macros.  NOTE: the warning
## we insert must not add any lines to the generated files).  Because
## of parallel builds, we have to make sure the target directories
## exist before pre-processing the file.  Asking make to do this
## analysis is not feasible.
$(DEST_DIR)/$($(PACKAGE_NAME)_DEST_DIR)/%.java: $(SRC_DIR)/$($(PACKAGE_NAME)_SRC_DIR)/%.Java $($(PACKAGE_NAME)_CPP_MACROS) 
	$(QUIET) set -e 
	@echo Preprocessing $<
#1. Make sure target directory actually exists
	$(QUIET) test -d $(dir $@) || $(MKDIR) $(dir $@)
#2. Remove existing target .java file
	$(QUIET) $(RM) $@
#3. Insert warning message into target
	$(QUIET) $(ECHO_CLEAN) \
		"/* WARNING: generated from $<. -*- buffer-read-only: t -*- */" > $@
#4. Do the actual pre-processing
	$(QUIET) $(PREPROCESS) $< >> $@ || ($(RM) $@; $(FAILURE))



## Make the current package actually dependent upon its dependent packages
.compile_$(PACKAGE_NAME): $(addprefix .compile_, $($(PACKAGE_NAME)_PKG_DEPENDENCIES))

## Compile the package.  Sends all of the .java files to the compiler
## all at once.  Depends on the .java files all being up to date (note
## that dependencies on other packages and directories are expressed
## separately in the previous two rules).
##
## The target file (.compile_<pkg>) is actually created.  If the
## compile fails, the error output is massaged so that it references
## the original .Java files (heh).
##
## A couple sanity checks are performed before and after the compile
## to make sure nothing subtle or bogus is happening.
##
.compile_$(PACKAGE_NAME): $($(PACKAGE_NAME)_CPPSOURCES) 
	$(QUIET) echo "Compiling $(subst .compile_,,$@) package ($(BUILD))"
# 1. sanity checks
	$(QUIET) if test -z "$($(subst .compile_,,$@)_SOURCES)"; then \
		echo "ERROR: Package $(subst .compile_,,$@) " \
			"contains no source files!?!?"; \
		$(FAILURE); \
	fi
	$(QUIET) if test -n "$($(subst .compile_,,$@)_EXTRA_CPP)"; then \
		echo "WARNING:  Package $(subst .compile_,,$@) destination " \
			"contains stale file(s): $($(subst .compile_,,$@)_EXTRA_CPP)"; \
	fi
	$(QUIET) test -z "$$JIKESPATH" || echo "WARNING: JIKESPATH is set.  This may cause compilation errors!"
# 2. remove empty target file
	$(QUIET) $(RM) $@ $(@)-output
# 3. run the compile, munging the output as necessary
	$(QUIET) if $(COMPILE_JAVA) $($(subst .compile_,,$@)_CPPSOURCES) 1> $(@)-output 2>&1 ; then \
		$(SED) -e 's/\.java/\.Java/g' \
	        	-e 's,$(DEST_DIR)/$(PACKAGE_PREFIX_DIR),$(SRC_DIR),g' $(@)-output; \
		touch $@;					  \
	else 							  \
		$(SED) -e 's/\.java/\.Java/g' \
		        -e 's,$(DEST_DIR)/$(PACKAGE_PREFIX_DIR),$(SRC_DIR),g' $(@)-output; \
		$(FAILURE);                                       \
	fi
# 4. post-compile sanity check for missing .class files (old jikes compiler may do this).
	$(QUIET) for cl in $($(subst .compile_,,$@)_MIN_CLASSES); do \
		if test ! -f $$cl; then \
			echo "ERROR: Expected class file $$cl is missing.  " \
				"Check .compile_*-output files."; \
			exit 11;		\
		fi; \
	done


## Add the compile target to the global list of package compile targets
COMPILATION_TARGETS+=.compile_$(PACKAGE_NAME)

###
### Cleanup targets
###

.PHONY: clean
clean:: clean_$(PACKAGE_NAME)

clean_%::
	@echo Cleaning $*
	$(RM) $(DEST_DIR)/$($(*)_DEST_DIR)/*.java
	$(RM) $(DEST_DIR)/$($(*)_DEST_DIR)/*.class

###
### Tags targets
###

.PHONY: tags

tags: tags_$(PACKAGE_NAME)

tags_%:
#	@echo Generating TAGS for $*
	$(QUIET) rm -f TAGS_$*
	$(QUIET) $(ETAGS) -o TAGS_$* --language=java $($(*)_SOURCES)

###
### Makefile debugging targets
###

.PHONY: mkdebug
mkdebug:: mkdebug_$(PACKAGE_NAME)

mkdebug_%::
	@echo 
	@echo "**** Targets for $* package"
	@echo "    $* Fullname    = $($(*)_FULLNAME)"
	@echo "    $* SRC_DIR     = $($(*)_SRC_DIR)"
	@echo "    $* DEST_DIR    = $($(*)_DEST_DIR)"
	@echo "    $* SOURCES     = $($(*)_SOURCES)"
	@echo "    $* SOURCES_GLOB= $($(*)_SOURCES_GLOB)"
	@echo "    $* CPP sources = $($(*)_CPPSOURCES)"
	@echo "    $* EXPECTED_CPP sources = $($(*)_EXPECTED_CPP)"
	@echo "    $* ACTUAL_CPP sources = $($(*)_ACTUAL_CPP)"
	@echo "    $* EXTRA_CPP sources = $($(*)_EXTRA_CPP)"
	@echo "    $* ACTUAL_CPP_GLOB = $($(*)_ACTUAL_CPP_GLOB)"

