/*
 * StatsReturnCapsule.Java
 *
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 *
 */
package JANOS_ANTSR_STATS_APP_PKG;

import JANOS_ANTSR_CORE_PKG.Protocol;
import JANOS_ANTSR_CORE_PKG.DataCapsule;
import JANOS_ANTSR_CORE_PKG.Xdr;
import JANOS_ANTSR_CORE_PKG.Node;
import JANOS_ANTSR_CORE_PKG.ByteArray;
import JANOS_ANTSR_CORE_PKG.NodeAddress;

import java.util.Vector;

/**
 * Stats return capsule.
 *
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Janos Ministry of Development</a>
 */
public class StatsReturnCapsule
	extends DataCapsule
{
	final private static byte[] MID = findMID(JANOS_ANTSR_STATS_APP_PKG_S + ".StatsReturnCapsule");
	final private static byte[] PID = findPID(JANOS_ANTSR_STATS_APP_PKG_S + ".StatsReturnCapsule");

	public int infosource;
	public int parentnode;
	public long capTime;
	public long latency;
	public int nodessentto;
	public Vector branches=new Vector();

	protected byte[] mid()
	{
		return MID;
	}

	protected byte[] pid()
	{
		return PID;
	}

	public int length()
	{
		return super.length()+Xdr.INT+Xdr.INT+Xdr.LONG+Xdr.LONG+Xdr.INT+Xdr.INT*nodessentto;
	}

	/**
	 * Return capsule looks like:<br>
	 * 4	Address of the actual source of the information<br>
	 * 4	Address of the node that requested stats from source node<br>
	 * 4	Time capsule this is a response to was sent<br>
	 * 4	Latency (0 if latency has yet to be figured)<br>
	 * 4	Number of nodes the source of this return sent to
	 * <p>
	 * The data in the datacapsule contains strings which specify the
	 * actual statistics of the node this is returning from
	 */
	public Xdr encode()
	{
		int i;
		Xdr xdr = super.encode();
		xdr.PUT(infosource);
		xdr.PUT(parentnode);
		xdr.PUT(capTime);
		xdr.PUT(latency);
		xdr.PUT(nodessentto);
		for (i=0;i<nodessentto;i++)
			xdr.PUT(((Integer)branches.elementAt(i)).intValue());
		return xdr;
	}

	public Xdr decode()
	{
		int i;
		Xdr xdr = super.decode();
		infosource=xdr.INT();
		parentnode=xdr.INT();
		capTime=xdr.LONG();
		latency=xdr.LONG();
		nodessentto=xdr.INT();
		for (i=0;i<nodessentto;i++)
			branches.addElement(new Integer(xdr.INT()));
		return xdr;
	}

	public boolean evaluate(Node n)
	{
		// Check the latency
		if (latency==0)
		{
			latency=(n.time()-capTime)/2;
		}
		if (n.getAddress()==getDst())
		{
			return n.deliverToApp(this,dpt);
		} else {
			return n.routeForNode(this,getDst());
		}
	}

	/**
	 * Print out the information that is contained in this capsule
	 */
	public void showInformation(Node n)
	{
		ByteArray buf=getData();
		Xdr xdr=new Xdr(buf,0);
		int i,num;
		n.log(0, "*** Recieved stats return capsule from "+
			NodeAddress.toString(infosource)+
			" with parent "+NodeAddress.toString(parentnode)+
			" with latency of "+latency+" containing:");
		num=xdr.INT();
		for (i=0;i<num;i++)
			n.log(0, "\t"+xdr.STRING()+": "+xdr.STRING());
	}

	public StatsReturnCapsule()
	{
	}

	public StatsReturnCapsule(int d)
	{
		setDst(d);
	}

	public StatsReturnCapsule(short sa, short da, int na, ByteArray d)
	{
		super(sa,da,na,d);
	}
}
