/* 
 *  $Id: ANEPFlow.Java,v 1.3 2000/08/28 20:54:24 tullmann Exp $
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 *
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 */
package JANOS_ANTSR_CORE_PKG;

import JANOS_ROUTE_PKG.RouteEntry;

IMPORT_DEBUG

/**
 * Forwards unrecognized ANEP packets.
 * An ANEP Flow will provide default handling for ANEP packets
 * that are not of a recognized type.  The associated Channel
 * is connected to the ANEPDispatcher's registerDefault method.
 * <p>
 * XXX Needs to be reformatted to the One True Janos Formatting
 * Standard.
 *
 * @history Originally ANEPChannel.Java
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Janos Ministry of Development</a>
 * @author David Wetherall
 */
public class ANEPFlow
{
  /**
   * Decode ANEP header and forward packet.  This is for non-ANTSR
   * ANEP packets.
   * <p>
   * Always returns null because no capsule is created.
   * 
   * @param xdr The external representation of the ANEP packet.
   * @return null
   */
  protected Capsule decode(Xdr xdr) throws Exception {

    // Make sure its an appropriate ANEP packet
    if (xdr.BYTE() != Capsule.ANEPVERSION)
      throw new Exception("Wrong ANEP version");

    boolean discard = (xdr.BYTE() & 0x80) != 0;
    short typeID = xdr.SHORT();
    short headerLen = xdr.SHORT();
    xdr.skip(Xdr.SHORT);     // Skip packet length field

    if (typeID != Capsule.ANTSRTYPE) {
      if (discard)
	throw new Exception("Unknown type discard");
      else {
	int dest = 0;
	while (xdr.index() < headerLen * Xdr.INT) {
	  short flagsAndType = xdr.SHORT();
	  boolean optPrivate = (flagsAndType & 0x8000) != 0;
	  boolean optDiscard = (flagsAndType & 0x4000) != 0;	
	  short optType = (short)(xdr.SHORT() & 0x3FFF);
	  short optLen = xdr.SHORT();
	  if (!optPrivate) {
	    if (optType == Capsule.DESTOPTTYPE) {
	      int scheme = xdr.INT();
	      if (scheme != Capsule.IPV4SCHEME)
		throw new Exception("Unknown addressing scheme");
	      else {
		dest = xdr.INT();
	      }
	    } else if (optType == Capsule.SRCOPTTYPE) {
	      xdr.skip((optLen - 1) * Xdr.INT);
	    } else if (optDiscard) 
	      throw new Exception("Unknown option discard");
	  } else {
	    xdr.skip((optLen - 1) * Xdr.INT);
	  }
	}

	panic("ANEPFlow not upgraded for getting routes and forwarding packets");

#if 0
	RouteEntry r = Node.getRoutes().get(dest);
	if (r != null) {
	  if (xdr.offset != 0) {
	    byte[] bytes = new byte[xdr.len];
	    System.arraycopy(xdr.buf, xdr.offset, bytes, 0, xdr.len);
	    Node.link.send(bytes, xdr.len, r.addr);
	  } else
	    Node.link.send(xdr.buf, xdr.len, r.addr);
	  throw new Exception("Unknown type forwarded");
	} else
	  throw new Exception("No route to destination while forwarding");
#endif
      }
    }
    // skip ANEP header
    xdr.skip(headerLen * Xdr.INT);
    return null;
  }

  /**
   * Construct an ANEPFlow.
   */
  public ANEPFlow()
  {
    panic("Dunno how to construct an ANEP Flow!");
  }
}
