/*
 * AppConfig.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_ANTSR_CORE_PKG;

IMPORT_DEBUG
IMPORT_TRACE

/**
 * A simple (classname, arguments) pair.  For some reason we
 * duplicate the className in the arguments passed.
 *
 * XXX this.className isn't really used.  Delete it.
 *
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Janos Ministry of Development</a>
 */
PACKAGE class AppConfig
{
	private final String className;
	private final String[] args;

	PACKAGE AppConfig(String className, String[] args, int argsStart)
	{
		require(className);
		require(args);
		require(argsStart >= 0);

		TRACE(Trace.ANTSR_APP, "Creating app cfg " +className+ " with args.length=" +args.length+ "; argsStart=" +argsStart);
		
		this.className = className;
		this.args = new String[args.length - argsStart + 1];
		this.args[0] = className;

		System.arraycopy(args, argsStart,
				 this.args, 1, this.args.length - 1);

		ensure(this.className);
		ensure(this.args);
	}

	public String className()
	{
		require(this.className);
		return this.className;
	}

	public String[] args()
	{
		require(this.args);
		return this.args;
	}

	public String toString()
	{
		String retval;
		int lpc;
		
		retval = "AppConfig[className="
			+ this.className
			+ "; args[]={";
		for( lpc = 0; lpc < this.args.length; lpc++ )
		{
			if( lpc > 0 )
				retval += ", ";
			retval += this.args[lpc];
		}
		retval += "}";
		retval += "]";

		return retval;
	}
}
