/*
 * Console.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_ANTSR_CORE_PKG;

import JANOS_IO_PKG.Log;

import JANOS_ANTSR_UTIL_PKG.LineListTokenizer;
import JANOS_ANTSR_UTIL_PKG.LineListException;

import java.net.ServerSocket;
import java.net.Socket;

import java.io.InputStreamReader;
import java.io.LineNumberReader;
import java.io.StringReader;
import java.io.PrintWriter;

IMPORT_DEBUG
IMPORT_TRACE

PACKAGE class Console
	implements java.lang.Runnable // XXX
{
	private static final int MAX_SOCKET_BACKLOG = 2;

	/**
	 * The socket this console listens on.
	 */
	private final ServerSocket socket;

	private final Node node;

	/**
	 * Create a Console listener on the given port.
	 * Blows up if the port is already in use, throwing
	 * an IOException.
	 * <p>
	 * Connections will not be handled until the <code>accept()</code>
	 * method is executed.  You'd best supply a new thread
	 * to handle them, too.
	 *
	 * @see #accept()
	 * @param port the port to use; zero for any free port.
	 * @exception IOException throws whatever errors ServerSocket might toss your way.
	 */
	PACKAGE Console(Node node, int port)
		throws java.io.IOException // XXX 
	{
		this.node = node;

		// May throw IOException:
		this.socket = new ServerSocket(port, MAX_SOCKET_BACKLOG);
	}

	/**
	 * Return the port this console server listens on
	 */
	PACKAGE int getPort()
	{
		return socket.getLocalPort();
	}

	public String toString()
	{
		return "Console[" +socket.toString()+ "]";
	}

	/**
	 * Set the connection handler function in motion
	 */
	public void run()
	{
		accept();
	}

	/**
	 * Accept and handle connections on this Console.
	 * <p>
	 * This call never returns.
	 */
	PACKAGE void accept()
	{
		ConsoleWorker worker;
		Socket client;

		while (true)
		{
			try
			{
				client = this.socket.accept();
				worker = new ConsoleWorker(this.node, client);
				this.node.threadStart(worker, "ConsoleWorker");
			}
			catch(Throwable th)
			{
				node.log(Log.WARNING,
					 "Exception on console: "
					 + th.getMessage());
				node.logStackTrace(Log.WARNING, th);
			}
		}
	}
}
