/*
 * Watchdog.Java
 *
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_ANTSR_CORE_PKG;

import JANOS_NODEOS_PKG.Status;

import JANOS_IO_PKG.Log;

import java.net.ServerSocket;
import java.net.Socket;

import java.io.InputStreamReader;
import java.io.LineNumberReader;
import java.io.PrintWriter;
import java.util.Date;

IMPORT_DEBUG
IMPORT_TRACE

/**
 * A simple watch dog class, it opens a network port and prints out little
 * messages to whoever has connected to it.
 */
PACKAGE class WatchDog
	implements java.lang.Runnable // XXX
{
	private static final int MAX_SOCKET_BACKLOG = 2;

	/**
	 * The worker class for the watch dog, this does the actual printing
	 * for each connected user.
	 */
	private class Worker
		implements Runnable
	{
		private PrintWriter out;

		public Worker(PrintWriter pw)
		{
			this.out = pw;
		}
		
		public void run()
		{
			while( true )
			{
				try
				{
					this.notice();
				}
				catch(java.io.IOException e)
				{
				}
			}
		}
		
		private void notice()
			throws java.io.IOException
		{
			this.out.println("Connected to WatchDog port on "
					 + node);
			while( true )
			{
				/* Print out something semi-meaningful */
				this.out.println("** node: "
						 + JANOS_NODEOS_PKG.Address.
						 toString(PrimordialNode.
							  getAddress())
						 + " time: "
						 + new Date()
						 + " **");
				this.out.flush();
				try
				{
					Thread.sleep(1000);
				}
				catch(InterruptedException e)
				{
				}
			}
		}
	}
	
	/**
	 * The socket this watch dog listens on.
	 */
	private final ServerSocket socket;

	private final Node node;

	/**
	 * Create a WatchDog listener on the given port.
	 * Blows up if the port is already in use, throwing
	 * an IOException.
	 * <p>
	 * Connections will not be handled until the <code>accept()</code>
	 * method is executed.  You'd best supply a new thread
	 * to handle them, too.
	 *
	 * @see #accept()
	 * @param port the port to use; zero for any free port.
	 * @exception IOException throws whatever errors ServerSocket might toss your way.
	 */
	PACKAGE WatchDog(Node node, int port)
		throws java.io.IOException // XXX 
	{
		this.node = node;

		// May throw IOException:
		this.socket = new ServerSocket(port, MAX_SOCKET_BACKLOG);
	}

	/**
	 * Return the port this watch dog server listens on
	 */
	PACKAGE int getPort()
	{
		return socket.getLocalPort();
	}

	public String toString()
	{
		return "WatchDog[" +socket.toString()+ "]";
	}

	public void run()
	{
		accept();
	}

	/**
	 * Accept and handle connections on this watch dog.
	 * <p>
	 * This call never returns.
	 */
	PACKAGE void accept()
	{
		Runnable worker;

		while( true )
		{
			try
			{
				PrintWriter pw;
				Socket client;
				
				client = this.socket.accept();
				pw = new PrintWriter(client.
						     getOutputStream());
				worker = new Worker(pw);
				client.getInputStream().close();
				this.node.threadStart(worker);
			}
			catch(java.io.IOException e)
			{
			}
		}
	}
}

