/* 
 * RouteTableReader.Java
 *  
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 *
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 */
package JANOS_ANTSR_ROUTE_PKG;

import JANOS_NODEOS_PKG.Address;
import JANOS_NODEOS_PKG.AddressFormatException;
import JANOS_NODEOS_PKG.NoSuchNodeAddress;

import JANOS_ROUTE_PKG.WriteableRouteTable;

import java.io.FileInputStream;
import java.io.StreamTokenizer;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.IOException;

IMPORT_DEBUG
IMPORT_TRACE

/**
 * Fill in a route table from a file.
 * <p>
 * This code was originally part of the "core" RouteTable.
 */
public class RouteTableReader
{
	/**
	 * Read a route table from a file.
	 *
	 * @param file the file to read
	 */
	public static void read(int thisNode, WriteableRouteTable rt, String file) 
		throws IOException, AddressFormatException, NoSuchNodeAddress
	{
		FileInputStream f = new FileInputStream(file); 
		StreamTokenizer st =
			new StreamTokenizer(new BufferedReader(new InputStreamReader(f)));

		assert(rt);

		st.resetSyntax();
		st.whitespaceChars(0, ' ');
		st.wordChars('0', '9');
		st.wordChars('.', '.');
		st.wordChars(':', ':');
		st.wordChars('-', '-');
		st.wordChars('A', 'Z');
		st.wordChars('a', 'z');
		st.commentChar('#');
		
		int src = 0;
		int dst = 0;
		int nxt = 0;
		String addr = null;
		
		TRACE(Trace.ANTSR_ROUTE_READ_TABLE, "Reading route table for node " +Address.toString(thisNode));

		while (st.nextToken() == StreamTokenizer.TT_WORD) {
			if (src == 0) {
				assert(dst == 0);
				assert(nxt == 0);
				assert(addr == null);
				src = Address.fromString(st.sval);
			}
			else if (dst == 0) {
				assert(nxt == 0);
				assert(addr == null);
				dst = Address.fromString(st.sval);
			} 
			else if (nxt == 0) {
				assert(addr == null);
				nxt = Address.fromString(st.sval);
			}
			else {
				assert(addr == null);
				addr = st.sval;
			
				TRACE(Trace.ANTSR_ROUTE_READ_TABLE, "Find route " +Address.toString(src)+ " -> " +Address.toString(dst)+ " via " +Address.toString(nxt)+ "(" +addr+ ")");

				// If this line is for a route from this node
				if (src == thisNode) 
				{
					rt.addRoute(dst, nxt, addr);
				}

				// Clear values for next line of route file
				src = 0;
				dst = 0;
				nxt = 0;
				addr = null;
			}
		}
		
		if (src != 0) 
			throw new IOException("premature end of input"); // xxx wrong exeception
		else if (st.ttype != StreamTokenizer.TT_EOF)
			throw new IOException("can't handle " + st.toString());// xxx wrong exeception
	}
}

