/*
 * AddressFactory.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_NODEOS_PKG;

/**
 * The Factory for creating Address objects.  Only one
 * Address factory is created, and it is created by
 * a static initalizer in Address.
 *
 * @see Address
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Janos Ministry of Development</a>
 */
public class AddressFactory
{
	/**
	 * Package-private constructor.  Only invoked in Address.
	 *
	 * @see Address.getAddressFactory
	 */
	PACKAGE AddressFactory()
	{
	}

	/**
	 * Construct a new Address object that associates
	 * the given, virtual "active address" with the
	 * physical address described by the addressSpec
	 * and its protocolSpec.
	 *
	 * @param anAddress the 32-bit IPv4-like active address.
	 * @param protocolSpec the protocol context for the addressSpec
	 * @param addressSpec the address specification
	 * @return a new Address object
	 * @allocation a single Address object
	 * @see InChannel#create()
	 * @see Address
	 */
	public Address newAddress(int anAddress,
				  String protocolSpec,
				  String addressSpec)
		throws NoSuchNodeAddress
	{
		return new Address(anAddress, protocolSpec, addressSpec);
	}
}
