/*
 * CutThroughChannel.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_NODEOS_PKG;

import java.io.IOException; // XXX

/**
 * Java wrapper for a CutThroughChannel.
 * 
 * @see InChannel
 * @see OutChannel
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Janos Ministry of Development</a>
 */
public final class CutThroughChannel
	extends NATIVE_CUTTHROUGHCHANNEL
{
	CHECK_NATIVE_NODEOS

	public static CutThroughChannel
	create(Flow flow,
	       DemultiplexKey classifier,
	       String inProtocolSpec,
	       String outProtocolSpec,
	       String inAddressSpec,
	       String outAddressSpec,
	       Address destAddr)
		throws IOException, // XXX
		PacketAddressFormatException,
		ProtocolSpecFormatException
	{
		// XXX this isn't sufficient protection for the rest of the kernel..
		if((flow == null)
		   || (classifier == null)
		   || (inProtocolSpec == null)
		   || (outProtocolSpec == null)
		   || (inAddressSpec == null)
		   || (outAddressSpec == null)
		   || (destAddr == null))
			throw new NullPointerException();

		return new CutThroughChannel(flow, classifier,
					     inProtocolSpec,
					     outProtocolSpec,
					     inAddressSpec,
					     outAddressSpec,
					     destAddr);
	}

	/**
	 * Constructors are private so I can do a validity check on the
	 * arguments.
	 */
	private CutThroughChannel(Flow flow,
				  DemultiplexKey classifier,
				  String inProtocolSpec,
				  String outProtocolSpec,
				  String inAddressSpec,
				  String outAddressSpec,
				  Address destAddr)
		throws IOException, // XXX
		PacketAddressFormatException,
		ProtocolSpecFormatException
	{
		super(flow.getNativeFlow(),
		      classifier.getNativeKey(),
		      inProtocolSpec,
		      outProtocolSpec,
		      inAddressSpec,
		      outAddressSpec,
		      destAddr);
	}
	
	/**
	 * Add a buffer to the channel
	 */
	public void addBuffer(BufferHandle bufh)
	{
		NATIVE_BUFFERHANDLE nb;

		require(bufh); // XXX 

		nb = bufh.getNativeBufH();
		super.native_addBuffer(nb);
	}

	/**
	 * Remove a buffer from the channel. 
	 * A new BufferHandle is allocated to point to the buffer.
	 * <p>
	 * XXX new a call that takes a BufferHandle and points it at
	 * the underlying buffer object.
	 *
	 * @allocation Allocates a new BufferHandle()
	 * @return A new BufferHandle pointing to the removed buffer.
	 */
	public BufferHandle remBuffer()
	{
		NATIVE_BUFFERHANDLE nbh;
		nbh = super.native_remBuffer();
		assert(nbh instanceof BufferHandle);
		return (BufferHandle)nbh;
	}
	
	PACKAGE void destroy()
	{
		TRACE(Trace.NODEOS_DESTROY, "Destroying " + this);
		super.destroy();
	}

	public String toString()
	{
		return super.toString();
	}
}


