/*
 * FlowHandle.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_NODEOS_PKG;

/**
 * Parent's handle on a child flow.  Returned from
 * Flow.newFlow().
 *
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Janos Ministry of Development</a>
 */
public final class FlowHandle
{
	/**
	 * The flow this handle is associated with.  Fixed
	 * when the FlowHandle is contstructed.
	 */
	PACKAGE final Flow flow;

	/**
	 * TermHandler executed when this flow is termainted.
	 * May be null.
	 * <p>
	 * XXX why is this here (and not in the Flow proper)?
	 */
	PACKAGE final Runnable termHandler;

	/**
	 * FlowHandles are created implicitly by a call
	 * to Flow.newFlow() and returned.
	 * <p>
	 * XXX Why is termHandler kept here?
	 *
	 * @param flow the flow this handle is associated with
	 * @param termHandler the handler to invoke when the flow is terminated.
	 */
	PACKAGE FlowHandle(Flow flow, Runnable termHandler)
	{
		this.flow = flow;
		this.termHandler = termHandler;
	}

	/**
	 * Destroy the Flow.  The termination handler in the
	 * parent flow will be invoked.
	 * <p>
	 * XXX the existence of this method means that FlowHandle objects
	 * must not be made visible to (other) untrusted Flows.
	 * <p>
	 * XXX this doesn't work yet.
	 */
	public void destroy()
	{
		NOTIMPLEMENTED("FlowHandle.destroy() is not yet implemented.");
	}

	public String toString()
	{
		return "FlowHandle[flow="
			+ flow
			+ "; termHandler="
			+ termHandler
			+ "]";
	}
}
