/*
 * InChannelHandle.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_NODEOS_PKG;


/**
 * A cross-process handle on a Java NodeOS InChannel.
 * <p>
 * Packets can be inserted into an InChannel via an InChannelHandle.
 * This class is not yet support by Moab, so it will only work
 * in the Pure Java version of the NodeOS.
 *
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Janos Ministry of Development</a>
 * @see InChannel
 */
public final class InChannelHandle
	extends NATIVE_INCHANNELHANDLE
{
	CHECK_NATIVE_NODEOS

	public InChannelHandle()
	{
		super();
		super.setChannel(null);
	}

	public InChannelHandle(InChannel inChan)
	{
		super();

		require(inChan); // XXX

		super.setChannel(inChan.getNativeInChan());
	}

	public void copy(InChannelHandle otherChan)
	{
		require(otherChan); // XXX

		super.copy((NATIVE_INCHANNELHANDLE)otherChan);
	}

	/**
	 * Push a buffer onto the associated InChannel.
	 * <p>
	 * XXX payload start is gross a hack.  Might not even be necessary?
	 * <p>
	 * XXX returns a swapped buffer (or null)?
	 */
	public boolean push(BufferHandle pkt, int payloadStart)
	{
		boolean rv;
		pkt.systemOwns();

		synchronized(this)
		{
			NATIVE_BUFFERHANDLE nb;
			nb = pkt.getNativeBufH();
			rv = super.push(pkt.getNativeBufH(), payloadStart);
		}

		pkt.userOwns();
		return rv;
	}

	public boolean pushv(BufferHandle[] bhl, int payloadStart)
	{
		throw new InternalError("InChannelHandle.pushv() not implemented!");
	}

	public String toString()
	{
		return super.toString();
	}
}

