/*
 * OutChannel.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_NODEOS_PKG;

import java.io.IOException;

import java.net.DatagramSocket;
import java.net.DatagramPacket;

/**
 * Java wrapper for a NodeOS OutChannel.
 *
 * @see CutThroughChannel
 * @see InChannel
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Janos Ministry of Development</a>
 */
public final class OutChannel
	extends NATIVE_OUTCHANNEL
{
	CHECK_NATIVE_NODEOS

	/**
	 * Create a new OutChannel.
	 */
	public static OutChannel create(Flow flow,
					String protocolSpec,
					String addressSpec)
		throws IOException,
		PacketAddressFormatException,
		ProtocolSpecFormatException
	{
		if ((flow == null)
		    || (protocolSpec == null)
		    || (addressSpec == null))
			throw new NullPointerException(/*XXX*/);

		return new OutChannel(flow, protocolSpec, addressSpec);
	}

	private OutChannel(Flow flow,
			   String protocolSpec,
			   String addressSpec)
		throws IOException,
		       PacketAddressFormatException,
		       ProtocolSpecFormatException
	{
		super(flow.getNativeFlow(), protocolSpec, addressSpec);
	}

	/**
	 * Send a single buffer.  This is a synchronous send.  The buffer
	 * will be owned by the system for the duration of the send,
	 * and then ownership will be restored to the caller after the send.
	 */
	public void send(BufferHandle bh)
	{
		NATIVE_BUFFERHANDLE nbh;

		bh.systemOwns();
		nbh = bh.getNativeBufH();
		super.send(nbh);
		bh.userOwns();
	}

	/**
	 * Send a single buffer.  Recycle the buffer (by making it
	 * available to the given InChannel after the send is complete.
	 * This is a synchronous send.  (It should probably be
	 * async.)
	 */
	public void sendAndRecycle(BufferHandle bh, InChannel dustbin)
	{
		NATIVE_BUFFERHANDLE nbh;

		bh.systemOwns();
		nbh = bh.getNativeBufH();
		super.sendAndRecycle(nbh, dustbin.getNativeInChan());
		bh.userOwns();
	}

	public void sendAsync(BufferHandle bh, DeliverFunc callback)
	{
		NOTIMPLEMENTED("OutChannel.sendAsync()");
	}
	
	/**
	 * Send an array of buffer handles.
	 *
	 * @allocation Allocates a new PacketBuffer and a new byte array
	 * and linearizes into that buffer.
	 */
	public void sendv(BufferHandle[] bh)
	{
		NOTIMPLEMENTED("OutChannel.sendv()");
	}

	PACKAGE void destroy()
	{
		TRACE(Trace.NODEOS_DESTROY, "Destroying " + this);
		super.destroy();
	}

	public String toString()
	{
		return super.toString();
	}
}
