/*
 * Thread.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_NODEOS_PKG;

/**
 * A Janos Thread. <code>java.lang.Thread</code> is not visible to
 * threads running on the NodeOS.
 * <p>
 * There are no visible Thread objects in Janos (all the methods
 * here are static ones).  New threads are created either implicitly
 * in response to packets, or explicitly by a call to
 * <code>Flow.threadStart()</code>.
 *
 * @see Flow.threadStart()
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Janos Ministry of Development</a>
 */
public class Thread
{
	CHECK_NATIVE_NODEOS

	private Thread()
	{
		// Cannot create a Java Thread object
	}

	/**
	 * The mighty directed yield.
	 * <p>
	 * The scheduler may honor the argument.  Or it may
	 * laugh at your suggestion and put the target thread
	 * in a black hole just to spite you.
	 * <p>
	 * Operates on the current thread, implicitly.
	 *
	 * @param yieldTo the thread that should be run in deference to the current thread.
	 */
	public static void yieldTo(Thread yieldTo)
	{
		//NATIVE_THREAD.yieldTo((yieldTo == null)?(null):(yieldTo.nThread));
		NATIVE_THREAD.yieldTo(null);
	}
	
	/**
	 * An undirected yield.
	 * <p>
	 * Equivalent to <code>yield(null)</code>.
	 */
	public static void yield()
	{
		yieldTo(null);
	}

	/**
	 * Put the current thread to sleep for at least
	 * the given amount of time.
	 * <p>
	 * Operates on the current thread, implicitly.
	 */
	public static void sleep(long millis, int nanos)
		throws InterruptedException
	{
		NATIVE_THREAD.sleep(millis, nanos);
	}

	/**
	 * Set the per-thread object pointer.
	 * Operates on the current thread, implicitly.
	 * <p>
	 * Returns the old object.
	 */
	public static Object setObject(Object obj)
	{
		return NATIVE_THREAD.setObject(obj);
	}

	/**
	 * Get the per-thread object pointer.
	 * Operates on the current thread, implicitly.
	 */
	public static Object getObject()
	{
		return NATIVE_THREAD.getObject();
	}
}
