/*
 * moab_BufferHandle.Java
 *
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_NODEOS_PKG;

/**
 * Look at the Pure Java Buffer Handle (pj_BufferHandle) for
 * documentation on individual methods.
 *
 * @see BufferHandle
 * @see pj_BufferHandle
 */
PACKAGE class moab_BufferHandle
{
	static 
	{
		ensureNativeSpace(moab_BufferHandle.class);
	}

	private static native void ensureNativeSpace(Class thisclass);

	PACKAGE static native int getMaxPacketLen();

	protected moab_BufferHandle(moab_BufferHandle other, int offset, int length)
	{
		init(other, offset, length);
	}

	private native void init(moab_BufferHandle other, int offset, int length);
	
	PACKAGE native int getLength();

	PACKAGE native int getOffset();

	PACKAGE native void reset();

	PACKAGE native void newBuffer(int offset, int length);

	PACKAGE native void cloneHandle(moab_BufferHandle src, int fromOffset, int fromLength);

	PACKAGE native void swapWith(moab_BufferHandle src, int fromOffset, int fromLength);

	PACKAGE native byte getByte(int index);

	PACKAGE native void setByte(int index, byte val);

	PACKAGE native int getInt(int index);

	PACKAGE native void setInt(int index, int val);

	PACKAGE native void getBytes(int srcPosition, byte[] dest,
				     int destPosition, int destLength);
	
	PACKAGE native void setBytes(int destPosition, byte[] src,
				     int srcPosition, int srcLength);

	/**
	 * Write info about this packet buffer into the given char array.
	 * Return the number of char array entries used to describe the buffer.
	 */
	PACKAGE native int dbgPacketSummary(char[] sbuf, int offset, int len);
	
	public String toString()
	{
		return "moab_BufferHandle["
			+ "; off=" + this.getOffset()
			+ "; len=" + this.getLength()
			+ "]";
	}
}
