/*
 * moab_Flow.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_NODEOS_PKG;

/**
 * Java wrapper for a NodeOS Flow.
 * 
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Janos Ministry of Development</a>
 */
PACKAGE class moab_Flow
{
	private final Runnable terminationHandler;

	CHECK_NATIVE_NODEOS

	static
	{
		ensureNativeSpace(moab_Flow.class);
	}

	private static native void ensureNativeSpace(Class thisclass);

	/**
	 * Initalize a new Flow.  See newFlow().
	 * <p>
	 * XXX implicitly starts threads running (blocked) on the thread pool.  Gak.
	 */
	PACKAGE moab_Flow(moab_Flow parentFlow,
			  moab_Credentials cred,
			  moab_ThreadPoolSpec threadPoolSpec,
			  moab_MemSpec memSpec,
			  Runnable termHandler)
	{
		this.terminationHandler = termHandler;
		init(parentFlow, cred, threadPoolSpec, memSpec);
	}

	private native void init(moab_Flow parentFlow,
				 moab_Credentials cred,
				 moab_ThreadPoolSpec threadPoolSpec,
				 moab_MemSpec memSpec);

	/**
	 * Called from native code (in moab_Flow.c) when a flows termination
	 * handler is invoked by Moab.
	 */
	PACKAGE void native_termHandler() 
	{
		if (terminationHandler != null)
			terminationHandler.run();
	}

	PACKAGE native void threadStart(java.lang.Runnable r, String name);

	PACKAGE native void destroy();

	PACKAGE native Object getObject();

	PACKAGE native void setObject(Object obj);

	public String toString()
	{
		return "moab_Flow[...]" ;
	}

	PACKAGE static native moab_Flow native_currentFlow();

#ifdef DEBUG
	PACKAGE native void setName(String name);
	
	PACKAGE native String getName();
#endif /*DEBUG*/	
}

