/*
 * moab_InChannel.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_NODEOS_PKG;

import java.io.IOException;

/**
 * See pj_InChannel for doc.
 *
 * @see pj_InChannel
 * @see InChannel
 * @see OutChannel
 * @see moab_OutChannel
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Janos Ministry of Development</a>
 */
PACKAGE class moab_InChannel
{
	CHECK_NATIVE_NODEOS

	static
	{
		ensureNativeSpace(moab_InChannel.class);
	}

	private static native void ensureNativeSpace(Class thisclass);

	PACKAGE moab_InChannel(moab_Flow flow,
			       moab_DemultiplexKey classifier,
			       String protocolSpec,
			       String addressSpec,
			       DeliverFunc deliverFunc)
		throws IOException,
		PacketAddressFormatException,
		ProtocolSpecFormatException
	{
		int rc;
		assert(protocolSpec.equals("if/ipv4/udp")); // XXX XXX

		AddressSpecPortAddr addr; // because its udp
		addr = new AddressSpecPortAddr(addressSpec);

		// XXX cannot handle packet source demux:
		require(addr.sourceAddr() == AddressSpecPortAddr.WILDCARD_ADDRESS);
		require(addr.sourcePort() == AddressSpecPortAddr.WILDCARD_PORT);

		// Convert the dest addr to an int
		byte[] b = addr.destAddr().getAddress();
		int destaddr = (b[0] << 24) | (b[1] << 16) | (b[2] << 8) | b[3];

		// XXX the "socket:" thing is a hack for Moab
		rc = init(flow, 
			  classifier,
			  "socket:"+protocolSpec,
			  null,
			  destaddr, addr.destPort(),
			  deliverFunc);
		assert(rc == 0); // XXX
	}
	
	private native int init(moab_Flow flow,
				moab_DemultiplexKey classifier,
				String protocolSpec,
				moab_ThreadPoolSpec tps,
				int destAddr, int destPort,
				DeliverFunc deliverFunc);

	PACKAGE native void native_addBuffer(moab_BufferHandle bufh);
	
	PACKAGE native moab_BufferHandle native_remBuffer();
	
	public String toString()
	{
		return "moab_InChannel[...]";
	}

	PACKAGE native void destroy();
	
	protected void finalize()
	{
		// XXX ??
	}
}
