/*
 * moab_OutChannel.Java
 *
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_NODEOS_PKG;

import java.io.IOException;

IMPORT_DEBUG

/**
 * Wrapper for Moab OutChannel
 *
 * @see CutThroughChannel
 * @see InChannel
 * @see OutChannel
 */
PACKAGE class moab_OutChannel
{
	CHECK_NATIVE_NODEOS

	static
	{
		ensureNativeSpace(moab_OutChannel.class);
	}

	private static native void ensureNativeSpace(Class thisclass);

	PACKAGE moab_OutChannel(moab_Flow flow,
				String protocolSpec,
				String addressSpec)
		throws IOException,
		       PacketAddressFormatException,
		       ProtocolSpecFormatException
	{
		int rc;

		assert(protocolSpec.equals("if/ipv4/udp")); // XXX XXX

		AddressSpecPortAddr addr;
		addr = new AddressSpecPortAddr(addressSpec);

		// XXX cannot handle spoofing at all:
		require(addr.sourceAddr() == AddressSpecPortAddr.WILDCARD_ADDRESS);
		require(addr.sourcePort() == AddressSpecPortAddr.WILDCARD_PORT);

		// Convert the dest addr to an int
		byte[] b = addr.destAddr().getAddress();
		int destaddr = (b[0] << 24) | (b[1] << 16) | (b[2] << 8) | b[3];

		// XXX the "socket:" thing is a hack for Moab
		rc = init(flow, "socket:"+protocolSpec,
			  destaddr, addr.destPort());
		assert(rc == 0); // XXX
	}

	private native int init(moab_Flow flow,
				String protocolSpec,
				/* int srcAddr, int srcPort,2*/
				int destAddr, int destPort);

	PACKAGE native void send(moab_BufferHandle bh);

	PACKAGE native void sendAndRecycle(moab_BufferHandle bh, moab_InChannel inChan);
	
	PACKAGE native void sendAsynch(moab_BufferHandle bh, DeliverFunc callback);

	PACKAGE native void sendv(moab_BufferHandle[] bhl);

	PACKAGE native void destroy();

	public String toString()
	{
		return "moab_OutChannel[...]";
	}
}
