/*
 * moab_Setup.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_NODEOS_PKG;

/**
 * Setup the Moab native NodeOS.
 *
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Janos Ministry of Development</a>
 */
PACKAGE class moab_Setup
{
	private static final String moabGlueLibraryName = "kaffenodeos";

	/**
	 * Initialize the Moab native glue layer.  Also, force the basic
	 * classes to be pre-loaded into the System.  This allows the
	 * native code to play nasty games with the apparent size of
	 * the class instances.
	 */
	PACKAGE static void initialize()
	{
		java.lang.System.loadLibrary(moabGlueLibraryName);

		// XXX This hack forces all of the low-level classes to be
		// fully loaded and parsed before any of their sub-classes
		// are.  XXX
		forceClass(JANOS_NODEOS_PKG.moab_BufferHandle.class);
		forceClass(JANOS_NODEOS_PKG.moab_Credentials.class);
		forceClass(JANOS_NODEOS_PKG.moab_CutThroughChannel.class);
		forceClass(JANOS_NODEOS_PKG.moab_Debug.class);
		forceClass(JANOS_NODEOS_PKG.moab_DemultiplexKey.class);
		forceClass(JANOS_NODEOS_PKG.moab_Flow.class);
		forceClass(JANOS_NODEOS_PKG.moab_InChannel.class);
		//forceClass(JANOS_NODEOS_PKG.moab_InChannelHandle.class);
		forceClass(JANOS_NODEOS_PKG.moab_MemSpec.class);
		forceClass(JANOS_NODEOS_PKG.moab_OutChannel.class);
		forceClass(JANOS_NODEOS_PKG.moab_Thread.class);
		//forceClass(JANOS_NODEOS_PKG.moab_ThreadPool.class);
		forceClass(JANOS_NODEOS_PKG.moab_ThreadPoolSpec.class);
	}

	/**
	 * Only exists to force .class resolutions to be made.
	 */
	public static void forceClass(Class cl)
	{
		require(!cl.isInterface());
		require(!cl.isArray());
		require(!cl.isPrimitive());
	}
}
