/*
 * moab_ThreadPoolSpec.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_NODEOS_PKG;

/**
 * Moab ThreadPoolSpec wrapper
 *
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Janos Ministry of Development</a>
 */
PACKAGE class moab_ThreadPoolSpec
{
	CHECK_NATIVE_NODEOS

	static
	{
		ensureNativeSpace(moab_ThreadPoolSpec.class);
	}

	private static native void ensureNativeSpace(Class thisclass);

	private native void init(int maxThreads,
				 moab_Thread[] threads,
				 int sched);

	/**
	 * A list of the moab threads pre-allocated for this ThreadPool.
	 * We probably don't need to keep a handle on this here.  We'll
	 * let it go for now...
	 * <p>
	 * (Probably keeps dead thread objects around longer than it should.)
	 */
	private final moab_Thread[] threads;

	public moab_ThreadPoolSpec(int maxThreads,
				   int stackSize,
				   ThreadScheduler /*XXX*/ ignored)
	{
		require(maxThreads > 1);
		require(stackSize >= 0);
		assert(maxThreads < 10); /* XXX just to be safe */

		threads = new moab_Thread[maxThreads];
		
		/* Just *create* the threads, they are not started. */
		for (int i = 0; i < maxThreads; i++)
		{
			threads[i] = new moab_Thread(/*XXX noname*/null,
						     stackSize);
		}
		
		init(maxThreads, threads, /* XXX */ 0);
	}
}
