/*
 * pj_CutThroughChannel.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_NODEOS_PKG;

import java.io.IOException;

/**
 * Pure Java implementation of a CutThroughChannel.
 * 
 * @see pj_InChannel
 * @see pj_OutChannel
 * @see CutThroughChannel
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Janos Ministry of Development</a>
 */
PACKAGE class pj_CutThroughChannel
{
	/**
	 * The OutChan this CutThrough forwards to.
	 */
	private final pj_OutChannel outTo;

	/**
	 * The InChan this CutThrough receives from.
	 */
	private final pj_InChannel inFrom;

	PACKAGE pj_CutThroughChannel(pj_Flow flow,
				     pj_DemultiplexKey classifier,
				     String inProtocolSpec,
				     String outProtocolSpec,
				     String inAddressSpec,
				     String outAddressSpec,
				     Address destAddr)
		throws IOException,
		PacketAddressFormatException,
		ProtocolSpecFormatException
	{
		// Caller did sanity checks on arguments

		outTo = new pj_OutChannel(flow, outProtocolSpec, outAddressSpec);
		// above might throw IOException

		// XXX can make this a method on pj_CutThroughChannel now.
		DeliverFunc f = new DeliverFunc() {
			public void receive(BufferHandle packet, int payloadStart)
				throws IOException
				{
					outTo.send(packet);
				}
		};

		inFrom = new pj_InChannel(flow,
					  classifier,
					  inProtocolSpec,
					  inAddressSpec,
					  f);
		
		flow.addChannel(this);
	}
	
	/**
	 * Add a buffer to the channel
	 */
	PACKAGE void native_addBuffer(pj_BufferHandle pb)
	{
		this.inFrom.native_addBuffer(pb);
	}

	/**
	 * Remove a buffer from the channel
	 */
	PACKAGE pj_BufferHandle native_remBuffer()
	{
		return this.inFrom.native_remBuffer();
	}
	
	PACKAGE void destroy()
	{
		TRACE(Trace.NODEOS_DESTROY, "Destroying " + this);
	}
}


