/*
 * pj_InChannelHandle.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_NODEOS_PKG;

/**
 * A cross-process handle on an pj_InChannel.  Packets can be inserted
 * into an pj_InChannel via a pj_InChannelHandle.
 *
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Janos Ministry of Development</a>
 * @see InChannelHandle
 * @see InChannel
 * @see pj_InChannel
 */
PACKAGE class pj_InChannelHandle
{
	/**
	 * This would be final, except that we need the InChannelHandle
	 * to be created first, and then initialized out of a CommSpace.
	 */
	private pj_InChannel inChan;

	PACKAGE pj_InChannelHandle()
	{
		setChannel(null);
	}

	PACKAGE void setChannel(pj_InChannel inChan)
	{
		// XXX reference counting on the target inChan?
		// XXX cross-process reference!
		this.inChan = inChan;
	}

	PACKAGE void copy(pj_InChannelHandle other)
	{
		setChannel(other.inChan);
	}

	/**
	 * Push a buffer onto the associated InChannel.
	 * <p>
	 * XXX currently deals with fragmented buffers internally.
	 * Should have a 'pushv' function that deals with buffer
	 * aggregates.  That our we should require that aggregates
	 * are linearized by the caller.
	 * <p>
	 * XXX payload start is gross a hack.  Might not even be necessary?
	 * <p>
	 * XXX returns a swapped buffer (or null)?
	 */
	PACKAGE boolean push(pj_BufferHandle pkt, int payloadStart)
	{
		boolean retval = true;

		TRACE(Trace.NODEOS_OUTGOING, "pj_InChannelHandle.push: " +Debug.dbgPacketSummary(pkt, 128));

		/*
		 * Push into a new BufferHandle in target process's
		 * context.
		 */
		inChan.handle(pkt, 0, payloadStart);
		return retval;
	}

	PACKAGE boolean pushv(BufferHandle[] bhl, int payloadStart)
	{
		throw new InternalError("pj_InChannelHandle.pushv() not implemented!");
	}

	public String toString()
	{
		return "pj_InChannelHandle[inChan=" +inChan+ "]";
	}
}

