/*
 * pj_PacketHandlerList.Java
 *
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_NODEOS_PKG;

import java.util.Vector;
import java.net.DatagramPacket;
import java.net.Socket;

/**
 * pj_PortHandleList is a helper class for protocol layers that is
 * used to map a number of PortAddress-based PacketHandlers to an
 * individual socket.
 *
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Janos Ministry of Development</a>
 */
PACKAGE class pj_PacketHandlerList
{
	private final Vector masks = new Vector();
	
	/**
	 * Find a packet handler that matches the given socket, we
	 * just check the source masks of each handler.
	 */
	PACKAGE pj_ProtocolLayer.PacketHandler matchMask(Socket socket)
	{
		pj_ProtocolLayer.PacketHandler ph, retval = null;
		int size = this.masks.size();
		int lpc;
		
		for ( lpc = 0; (lpc < size) && (retval == null); lpc++ )
		{
			ph = (pj_ProtocolLayer.PacketHandler)this.masks.
				elementAt(lpc);
			if ( ((pj_PortAddressMask)ph.source).matchSocket(socket) )
				retval = ph;
		}
		return retval;
	}

	/**
	 * Find a packet handler that matches the given packet, we
	 * just check the source masks of each handler.
	 */
	PACKAGE pj_ProtocolLayer.PacketHandler matchMask(DatagramPacket dp)
	{
		pj_ProtocolLayer.PacketHandler ph, retval = null;
		int size = this.masks.size();
		int lpc;
		
		for ( lpc = 0; (lpc < size) && (retval == null); lpc++ )
		{
			ph = (pj_ProtocolLayer.PacketHandler)this.masks.
				elementAt(lpc);
			if ( ((pj_PortAddressMask)ph.source).matchPacket(dp) )
				retval = ph;
		}
		return retval;
	}

	/**
	 * Add a packet handler to this socket.
	 */
	PACKAGE void addHandler(pj_ProtocolLayer.PacketHandler ph)
	{
		int lpc, size;
		
		synchronized (this.masks)
		{
			size = this.masks.size();
			for ( lpc = 0; lpc < size; lpc++ )
			{
				pj_ProtocolLayer.PacketHandler elem =
					(pj_ProtocolLayer.PacketHandler)this.
					masks.elementAt(lpc);
				
				if ( ((pj_PortAddressMask)ph.source).priority() >=
				     ((pj_PortAddressMask)elem.source).priority() )
					break;
			}
			this.masks.insertElementAt(ph, lpc);
		}
	}
	
	/**
	 * Remove a packet handler from this socket.
	 */
	PACKAGE boolean remHandler(pj_ProtocolLayer.PacketHandler ph)
	{
		boolean retval = false;
		
		synchronized (this.masks)
		{
			this.masks.removeElement(ph);
			if ( this.masks.size() == 0 )
			{
				retval = true;
			}
		}
		return( retval );
	}
}
