/*
 * pj_PortAddressMask.Java
 *
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_NODEOS_PKG;

import java.net.Socket;
import java.net.InetAddress;
import java.net.DatagramPacket;
import java.net.UnknownHostException;
import java.util.StringTokenizer;
import java.util.NoSuchElementException;

/**
 * Mask for PortAddresses.
 *
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Janos Ministry of Development</a>
 */
PACKAGE class pj_PortAddressMask
	implements pj_ProtocolLayer.PacketAddressMask
{
	PACKAGE pj_InetAddressMask srcMask;
	PACKAGE int srcPort = -1;
	PACKAGE int destPort = -1;
	PACKAGE pj_InetAddressMask destMask;
	
	/**
	 * Parses UDP address mask strings.  The format is:
	 *  "<src address>:<dest port>:<dest address>:<src port>
	 */
	public void parseAddressMask(String addr)
		throws PacketAddressFormatException
	{
		StringTokenizer st = new StringTokenizer(addr, ":");
		String comp;
		
		try
		{
			this.srcMask =
				new pj_InetAddressMask(st.nextToken());
			comp = st.nextToken();
			if( !comp.equals("*") )
				this.destPort = Integer.parseInt(comp);
			this.destMask =
				new pj_InetAddressMask(st.nextToken());
			comp = st.nextToken();
			if( !comp.equals("*") )
				this.srcPort = Integer.parseInt(comp);
		}
		catch(UnknownHostException e)
		{
			throw new PacketAddressFormatException
				("Bad host (" +destMask+ ") in spec: " +addr);
		}
		catch(NoSuchElementException e)
		{
		}
	}

	/**
	 * Check for a match between our mask and the datagram packet
	 */
	PACKAGE boolean matchPacket(DatagramPacket dp)
	{
		boolean retval = false;

		if ( this.srcMask.match(dp.getAddress()) &&
		     ((this.srcPort == -1) ||
		      (dp.getPort() == this.srcPort)) )
		{
			retval = true;
		}
		return retval;
	}

	PACKAGE boolean matchSocket(Socket sock)
	{
		boolean retval = false;

		if ( this.srcMask.match(sock.getInetAddress()) &&
		     ((this.srcPort == -1) ||
		      (sock.getPort() == this.srcPort)) )
		{
			retval = true;
		}
		return retval;
	}


	PACKAGE int priority()
	{
		int retval;

		retval = this.destMask.priority() +
			this.srcMask.priority();
		if( this.srcPort != -1 )
			retval++;
		if( this.destPort != -1 )
			retval++;
		return retval;
	}
		
	public int hashCode()
	{
		return this.srcMask.hashCode() +
			this.srcPort +
			this.destPort +
			this.destMask.hashCode();
	}
		
	public boolean equals(Object o)
	{
		boolean retval = false;
			
		if( o instanceof pj_PortAddressMask )
		{
			pj_PortAddressMask uam = (pj_PortAddressMask)o;

			if( uam.srcMask.equals(this.srcMask) &&
			    uam.destMask.equals(this.destMask) &&
			    (uam.srcPort == this.srcPort) &&
			    (uam.destPort == this.destPort) )
			{
				retval = true;
			}
		}
		return( retval );
	}
		
	public String toString()
	{
		return "pj_PortAddressMask[srcAddrMask="
			+ this.srcMask
			+ "; srcPort="
			+ this.srcPort
			+ "; destPort="
			+ this.destPort
			+ "; destAddrMask="
			+ this.destMask
			+ "]";
	}
}

