/*
 * pj_StdinStreamLayer.Java
 *
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_NODEOS_PKG;

/**
 * pj_StdinStreamLayer is a protocol layer implementation that parses
 * out ANEP packets that are coming over Stdin.
 *
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Janos Ministry of Development</a>
 */
PACKAGE class pj_StdinStreamLayer
	extends pj_ANEPStreamLayer
{
	private boolean active = false;
	
	PACKAGE pj_StdinStreamLayer()
	{
	}

	/**
	 * There's no real address to apply a mask too so we just accept
	 * everything.
	 */
	private static class DummyAddressMask
		implements pj_ProtocolLayer.PacketAddressMask
	{
		public void parseAddressMask(String addr)
		{
		}

		public int hashCode()
		{
			return 0;
		}

		public boolean equals(Object o)
		{
			boolean retval = false;
			
			if ( o instanceof DummyAddressMask )
				retval = true;
			return retval;
		}
	}

	/**
	 * A StreamReader subclass that uses Stdin as the InputStream.
	 */
	private static class StdinStreamReader
		extends pj_ANEPStreamLayer.StreamReader
	{
		private PacketHandler ph;

		PACKAGE StdinStreamReader(PacketHandler ph)
		{
			// XXX wrong layer to create BufferHandle()
			super(System.in, BufferHandle.create()); 

			this.ph = ph;
		}

		/**
		 * Just pass it off to the packet handler.
		 */
		protected boolean handlePacket(pj_BufferHandle bh)
		{
			return this.ph.handle(bh);
		}
	}

	PACKAGE String getID()
	{
		return "stdin";
	}

	/**
	 * You can't send with this protocol layer so, just panic for now.
	 */
	PACKAGE AddressSpec createAddress(String addr)
		throws PacketAddressFormatException
	{
		panic("Stdin is a receive only layer!");
	}

	PACKAGE PacketAddressMask createAddressMask()
	{
		return new DummyAddressMask();
	}

	PACKAGE void attachHandler(PacketHandler ph)
	{
		/*
		 * We only need a single reader thread so just attach it once
		 */
		synchronized (this)
		{
			if ( !active )
			{
				java.lang.Thread thread;
				StreamReader sr;
				
				sr = new StdinStreamReader(ph);
				thread = new java.lang.Thread(sr);
				thread.setDaemon(true);
				thread.start();
				active = true;
			}
		}
	}

	PACKAGE void detachHandler(PacketHandler ph)
	{
	}

	/**
	 * You can't send with this protocol layer so, just panic for now.
	 */
	PACKAGE void attachGenerator(PacketGenerator pg)
	{
		panic("Stdin is a receive only layer!");
	}

	/**
	 * Synchronous send a packet, blocking until the send
	 * completes.
	 */
	PACKAGE void synchronousSend(PacketGenerator pg,
				     pj_BufferHandle bh)
	{
		panic("Stdin is a receive only layer!");
	}

	/**
	 * You can't send with this protocol layer so, just panic for now.
	 */
	PACKAGE void generatorReady(PacketGenerator pg)
	{
		panic("Stdin is a receive only layer!");
	}
	
	/**
	 * You can't send with this protocol layer so, just panic for now.
	 */
	PACKAGE void detachGenerator(PacketGenerator pg)
	{
		panic("Stdin is a receive only layer!");
	}
}
