/*
 * StdinTest.Java
 *
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

package JANOS_TESTS_PKG;

import JANOS_NODEOS_PKG.FlowRun;
import JANOS_NODEOS_PKG.Flow;
import JANOS_NODEOS_PKG.MemSpec;
import JANOS_NODEOS_PKG.Node;
import JANOS_NODEOS_PKG.InChannel;
import JANOS_NODEOS_PKG.DeliverFunc;
import JANOS_NODEOS_PKG.DemultiplexKey;
import JANOS_NODEOS_PKG.BufferHandle;
import JANOS_NODEOS_PKG.Address;
import JANOS_NODEOS_PKG.AddressFactory;
import JANOS_NODEOS_PKG.NoSuchNodeAddress;
import JANOS_UTIL_PKG.FlowBase;

import java.io.IOException;

public class StdinTest
	extends FlowBase
{
	private static final boolean VERBOSE = false;

	public StdinTest()
	{
	}

	static final int port = 33001;
	static final int count = 99;

	public void start()
	{
		Node.println("Started StdinTest");

		try 
		{
			start1();
		}
		catch(Throwable e)
		{
			e.printStackTrace();
			throw new RuntimeException("test failed");
		}
	}

	public void start1()
		throws Exception
	{
		// Get address factory because we want to create
		// nodes address with specific "physical" mappings.

		AddressFactory af = Address.getAddressFactory();
		Address addr = af.newAddress(1,
					     "stdin",
					     "*:*:localhost");

		InChannel in = makeInChannel(addr);
		// XXX We have to add 100, else it will hemorrhage most of them
		in.addBuffers(100);
	}

	private class StdinTest_Deliver
		implements DeliverFunc
	{
		PACKAGE InChannel myInChan;

		private final Address inAddr;

		public StdinTest_Deliver(Address inAddr)
		{
			this.inAddr = inAddr;
		}
		
		
		// XXX this only works if packets are
		// reliable and ordered
		
		public void receive(BufferHandle packet, int _)
		{
			try
			{
				receive1(packet);
			}
			catch(IOException e)
			{
				// XXX msg
				e.printStackTrace();
			}
		}
		
		public void receive1(BufferHandle packet)
			throws IOException
		{
			final int headerLength = 40;
			int i = packet.getInt(headerLength);
			
			if (VERBOSE)
				Node.println("Got a packet. i=" +i);

			if(i < count)
			{
				Node.println("port "
					     + inAddr.getProtocolSpec()
					     + ";"
					     + inAddr.getAddressSpec()
					     + " recv:" + i);
				myInChan.addBuffer(packet);
			}
			
			if(i >= count)
			{
				if (VERBOSE)
					Node.println("Shutting the test down.");
				StdinTest.this.flow.destroy();
			}
		}
	}

	private static class StdinDemultiplexKey
		extends DemultiplexKey
	{
		PACKAGE StdinDemultiplexKey(Flow flow)
		{
			super(flow);
			/* Just use the builtin matcher */
			this.matchANEP(0, (byte)1, true, (short)0);
		}
		
	}
	
	private InChannel makeInChannel(Address inAddr)
		throws Exception
	{
		InChannel retval;

		StdinTest_Deliver deliver = new StdinTest_Deliver(inAddr);

		retval = InChannel.create(flow,
					  new StdinDemultiplexKey(this.flow),
					  inAddr.getProtocolSpec(),
					  inAddr.getAddressSpec(),
					  deliver);

		deliver.myInChan = retval;

		return retval;
	}
}

