/*
 * TestMain.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_TESTS_PKG;

import JANOS_NODEOS_PKG.Flow;
import JANOS_NODEOS_PKG.Main;
import JANOS_NODEOS_PKG.Node;
import JANOS_NODEOS_PKG.Address;
import JANOS_NODEOS_PKG.AddressFactory;
import JANOS_NODEOS_PKG.CommSpaceHandle;
import JANOS_NODEOS_PKG.CommSpaceElement;

/**
 * A class to provide a main() for testing.  It will startup up a TestFlow
 * and pass it it's command line arguments.
 */
public class TestMain
{

	/**
	 * The root flow
	 */
	PACKAGE static Flow rootFlow;

	/**
	 * The test flow that gets started to run the tests in
	 */
	PACKAGE static TestFlow testFlow;

	/**
	 * The test node for the tests
	 */
	PACKAGE static JANOS_ANTSR_CORE_PKG.Node testNode;
	
	/**
	 * The address factory from nodeos, this can only be gotten once
	 * so we have to grab it and export it for everyone else
	 */
	private static AddressFactory addrFactory;

	/**
	 * Encoding of the build environment
	 */
	private static final int environment =
#ifdef DEBUG
		Tester.ENV_DEBUG |
#endif
#ifdef NDEBUG
		Tester.ENV_NDEBUG |
#endif
#ifdef EXPOSED
		Tester.ENV_EXPOSED |
#endif
		0;

	/**
	 * Startup the flow and setup any other references
	 */
	private static void setupObjects(String mainArgs[])
	{
		String args[] = new String[mainArgs.length + 1];

		args[0] = JANOS_TESTS_PKG_S + ".TestFlow";
		System.arraycopy(mainArgs, 0, args, 1, mainArgs.length);
		Main.main(args);
		TestMain.addrFactory = Address.getAddressFactory();
	}

	/**
	 * Return rootFlow
	 */
	public static Flow getRootFlow()
	{
		return TestMain.rootFlow;
	}

	/**
	 * Return the test flow object
	 */
	public static TestFlow getTestFlow()
	{
		return TestMain.testFlow;
	}

	/**
	 * Return the test node object
	 */
	public static JANOS_ANTSR_CORE_PKG.Node getNode()
	{
		return TestMain.testNode;
	}

	/**
	 * Return the address factory
	 */
	public static AddressFactory getAddressFactory()
	{
		return TestMain.addrFactory;
	}

	/**
	 * Get the build environment encoding
	 */
	public static int getEnvironment()
	{
		return TestMain.environment;
	}

	/**
	 * Print the usage for this classes' main()
	 */
	private static void usage()
	{
		Node.println("usage: "
			     + Tester.class.getName()
			     + " <test-classes>");
	}

	public static void main(String args[]) throws Exception
	{
		Tester.verbose = true;
		if( args.length > 0 )
		{
			TestMain.setupObjects(args);
		}
		else
		{
			TestMain.usage();
			System.exit(1);
		}
	}

}
