/*
 * WatchDog.Java
 *
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

package JANOS_TESTS_PKG;

/**
 * A simple watch dog thread.  It waits for a given length of time and then
 * interrupts the target thread, unless it gets interrupted beforehand.
 */
public class WatchDog extends Thread
{
	/**
	 * The amount of time to sleep before interrupting thread
	 */
	private long time;

	/**
	 * The thread to watch, and interrupt if time expires before we're
	 * interrupted
	 */
	private Thread thread;

	/**
	 * Construct a watch dog thread that watches a thread for ti time
	 *
	 * @param ti The length of time to sleep before interrupting the thread
	 * @param th The thread to watch
	 */
	public WatchDog(long ti, Thread th)
	{
		this.time = ti;
		this.thread = th;
	}

	/**
	 * Sleep for time milliseconds and then call interrupt on thread if
	 * we aren't interrupted before hand.  An interrupt called on our
	 * thread will cause make the thread quit quietly.
	 */
	public void run()
	{
		try
		{
			Thread.sleep(this.time);
			System.err.println("Watch dog timed out!");
			this.thread.interrupt();
			System.err.println("Interrupted thread "
					   + this.thread);
		}
		catch(InterruptedException e)
		{
		}
	}
}
