/*
 * CapsuleTester.Java
 *
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

package JANOS_TESTS_ANTSR_CORE_PKG;

import JANOS_TESTS_PKG.*;
import JANOS_ANTSR_CORE_PKG.*;
import JANOS_NODEOS_PKG.CommQueue;

public class CapsuleTester extends Tester
{
	
	private static class TestCapsule extends Capsule
	{
		final private static byte[] MID = new byte[16];
		final private static byte[] PID = new byte[16];
		
		protected byte[] mid()
		{
			return MID;
		}
		
		protected byte[] pid()
		{
			return PID;
		}

		public boolean evaluate(Node node)
		{
			return true;
		}
	}
	
	public void testUnit() throws TestFailedException
	{
		CommQueue cq = new CommQueue();
		int addr = 0xf0f0f0f0;
		TestCapsule tc, tc2;
		Xdr xdr;
		
		this.enterSection(ENV_NONE, "Constructor");
		tc = new TestCapsule();
		this.enterSection(ENV_NONE, "setters/getters");
		tc.setDst(addr);
		this.test(tc.getDst() == addr);
		this.enterSection(ENV_EXPOSED, "private stuff");
#ifdef EXPOSED
		xdr = new Xdr(tc.length());
		tc.xdr = xdr;
		tc.encode();
		xdr.index = 16; // Skip the MID
		try
		{
			tc2 = (TestCapsule)Capsule.
				create(TestCapsule.class, xdr);
			this.test(tc2.getDst() == addr);
		}
		catch(Exception e)
		{
			throw new TestFailedException(this);
		}
		this.test(tc.length() == Capsule.HEADERLENGTH);
		tc.setPrevious(addr);
		this.test(tc.getPrevious() == addr);
		tc.prime(addr, 5);
		this.test(tc.getTTL() == 5);
		this.test(tc.getSrc() == addr);
		tc2 = new TestCapsule();
		tc2.prime(tc);
		this.test(tc.getTTL() == 4);
		this.test(tc2.getSrc() == addr);
		this.test(tc2.getTTL() == 4);
		tc.enqueueCapsule(cq);
		// XXX dequeue it someday...
#endif
	}
}
