/*
 * ProtocolTester.Java
 *
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

package JANOS_TESTS_ANTSR_CORE_PKG;

import JANOS_TESTS_PKG.*;
import JANOS_ANTSR_CORE_PKG.*;

public class ProtocolTester extends Tester
{
	private static class TestCapsule extends Capsule
	{
		final private static byte[] MID = new byte[16];
		final private static byte[] PID = new byte[16];
		
		protected byte[] mid()
		{
			return MID;
		}
		
		protected byte[] pid()
		{
			return PID;
		}

		public boolean evaluate(Node node)
		{
			return true;
		}
	}
	
	private static class TestProtocol extends Protocol
	{
		public TestProtocol()
			throws Exception
		{
			startProtocolDefn();
			startGroupDefn();
			addCapsule(JANOS_TESTS_ANTSR_CORE_PKG_S +
				   ".ProtocolTester$TestCapsule");
			endGroupDefn();
			endProtocolDefn();
		}
	}
	
	public void testUnit() throws TestFailedException
	{
		TestProtocol tp;
		
		this.enterSection(ENV_NONE, "Constructor");
		try
		{
			tp = new TestProtocol();
		}
		catch(Exception e)
		{
			throw new TestFailedException(this, e);
		}
		this.enterSection(ENV_EXPOSED, "privates");
#ifdef EXPOSED
		Protocol.CapsuleClassData ccd;
		Protocol.CodeGroupData cgd;
		
		this.test("group size", tp.groups.size() == 1);
		cgd = ((Protocol.CodeGroupData)tp.groups.elementAt(0));
		this.test("cgd active", cgd.active == true);
		this.test("classes size", cgd.classes.size() == 1);
		ccd = (Protocol.CapsuleClassData)cgd.classes.elementAt(0);
		this.test("ccd name",
			  ccd.name.equals(JANOS_TESTS_ANTSR_CORE_PKG_S +
					  ".ProtocolTester$TestCapsule"));
#endif
	}
}
