/*
 * DynamicRoutingTester.Java
 *
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

package JANOS_TESTS_ANTSR_ROUTE_PKG;

#ifdef EXPOSED
import JANOS_ANTSR_CORE_PKG.PrimordialNode;
#endif
import JANOS_ANTSR_CORE_PKG.NodeAddress;
import JANOS_ANTSR_CORE_PKG.RouteTable;
import JANOS_ANTSR_CORE_PKG.Application;
import JANOS_NODEOS_PKG.AddressFormatException;
import JANOS_ROUTE_PKG.RouteEntry;

public class DynamicRoutingTester extends Application // Tester
{
	/**
	 * Ignore failures until the last check
	 */
	private boolean ignoreFail;
	
	//public void testUnit() throws TestFailedException
	/**
	 * Run the test
	 */
	public void start()
	{
		int tries;
		//this.enterSection(ENV_EXPOSED, "Constructor");
#ifdef EXPOSED
		// Need to wait for dynamic routing to settle down
		ignoreFail=true;
		for (tries=0;tries<=30;tries++)
		{
			try
		       	{
				synchronized (this)
				{
					this.wait(1000);
				}
			} catch (InterruptedException _) {
				tries--;
			}
			if (tries==30)
				ignoreFail=false;
			if (checkRoutes())
			{
				thisNode().log(0,"The test succeeded");
				break;
			}
		}
#else
		thisNode().log(0,"This test requires exposed build");
#endif
		thisNode().shutdown();
	}

	/**
	 * Fails the test and causes the node to exit with an error if
	 * ignoreFail is false.  If ignoreFail is true, then this does
	 * nothing.
	 */
	private void testFail(String msg)
	{
		if (ignoreFail)
			return;
		//throw new TestFailedException(msg);
		thisNode().log(0,"The test failed");
		thisNode().log(0,msg);
		thisNode().shutdown();
	}

#ifdef EXPOSED
	/**
	 * Checks if the routes are setup correctly
	 */
	private boolean checkRoutes()
	{
		RouteTable table;
		RouteEntry entry;
		int r2,r3;
		// Check the routing table
		table=PrimordialNode.routeTable;
		if (table==null)
			{ testFail("No routing table"); return false; }
		try
		{
			r2=NodeAddress.fromString("18.31.12.2");
			r3=NodeAddress.fromString("18.31.12.3");
		} catch (AddressFormatException _) {
			testFail("AddressFormatException");
			return false;
		}
		entry=table.getRoute(r2);
		if (entry==null)
			{ testFail("No route to 18.31.12.2"); return false; }
		if (entry.getLogicalDest()!=r2||entry.getLogicalNext()!=r2)
			{ testFail("Bad route to 18.31.12.2"); return false; }
		entry=table.getRoute(r3);
		if (entry==null)
			{ testFail("No route to 18.31.12.3"); return false; }
		if (entry.getLogicalDest()!=r3||entry.getLogicalNext()!=r2)
			{ testFail("Bad route to 18.31.12.3"); return false; }
		return true;
	}
#endif
}

