/*
 * CommQueueTester.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

package JANOS_TESTS_NODEOS_PKG;

import JANOS_TESTS_PKG.*;
import JANOS_NODEOS_PKG.*;

public class CommQueueTester extends Tester
{
	private TestFailedException consumerException = null;
	private final int exitValue = 10;

	/**
	 * Runnable class that consumes data
	 */
	private class Consumer implements Runnable
	{
		private CommQueue cq;
		
		public Consumer(CommQueue cq)
		{
			this.cq = cq;
		}
		
		public void run()
		{
			CommSpaceElement elem;
			
			try
			{
				boolean done = false;
				
				while( !done )
				{
					elem = cq.waitToTake();
					if( elem.getInt() == exitValue )
						done = true;
				}
			}
			catch(InterruptedException e)
			{
				CommQueueTester cqt =
					CommQueueTester.this;
				
				cqt.consumerException =
					new TestFailedException(cqt);
			}
		}
	}
	
	public void testUnit() throws TestFailedException
	{
		final CommQueue cq = new CommQueue();
		java.lang.Thread watchDog;
		CommSpaceElement cse;
		int lpc;

		this.enterSection(ENV_NONE, "Queue operations");

		/* enqueue some elements to be taken */
		int elements[] =
		{
			1,
			2,
			3,
			4
		};
		for( lpc = 0; lpc < elements.length; lpc++ )
		{
			cse = new CommSpaceElement(elements[lpc]);
			cq.enqueue(cse);
		}

		/* take the elements, use watch dog so we don't stall */
		watchDog = new WatchDog(10000,
					java.lang.Thread.currentThread());
		watchDog.start();
		try
		{
			for( lpc = 0; lpc < elements.length; lpc++ )
			{
				cse = cq.waitToTake();
				this.test(cse.getInt() == elements[lpc]);
			}
		}
		catch(InterruptedException e)
		{
			throw new TestFailedException(this);
		}
		watchDog.interrupt();

		java.lang.Thread consumer;

		/* start a consumer thread and enqueue some elements for it */
		consumer = new java.lang.Thread(new Consumer(cq));
		try
		{
			consumer.start();
			for( lpc = 0; lpc <= exitValue; lpc++ )
			{
				/* sleep so the consumer can do something */
				java.lang.Thread.sleep(50);
				cq.enqueue(new CommSpaceElement(lpc));
			}
			consumer.join(1000);
			if( consumer.isAlive() )
			{
				consumer.interrupt();
				throw new TestFailedException(this);
			}
		}
		catch(InterruptedException e)
		{
			consumer.interrupt();
			throw new TestFailedException(this);
		}
		if( this.consumerException != null )
			throw this.consumerException;
	}
}
