/*
 * CommSpaceElementTester.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */

package JANOS_TESTS_NODEOS_PKG;

import JANOS_TESTS_PKG.*;
import JANOS_NODEOS_PKG.*;

public class CommSpaceElementTester extends Tester
{
	private static final int INT_TYPE = 1;
	private static final int CHAR_ARRAY_TYPE = 2;
	private static final int BYTE_ARRAY_TYPE = 3;
	private static final int IN_CHANNEL_TYPE = 4;
	private static final int ADDRESS_TYPE = 5;
	private static final int BUFFER_HANDLE_TYPE = 6;
	private static final int COMM_SPACE_HANDLE_TYPE = 7;
	private static final int COMM_QUEUE_TYPE = 8;
	private static final int COMM_MONITOR_TYPE = 9;

	/**
	 * Verify that a CommSpaceElement has the correct type, we basically
	 * just want to make sure that none of the get methods returns a value
	 * of a completely different type.
	 *
	 * @param cse The CommSpaceElement with the value to test
	 * @param realType One of the above identifiers for indicating the type
	 */
	private void verifyType(CommSpaceElement cse, int realType)
		throws TestFailedException
	{
		try
		{
			cse.getInt();
			if( realType != INT_TYPE )
				throw new TestFailedException(this);
		}
		catch(ClassCastException e)
		{
			if( realType == INT_TYPE )
				throw new TestFailedException(this);
		}
		try
		{
			cse.getCharArray();
			if( realType != CHAR_ARRAY_TYPE )
				throw new TestFailedException(this);
		}
		catch(ClassCastException e)
		{
			if( realType == CHAR_ARRAY_TYPE )
				throw new TestFailedException(this);
		}
		try
		{
			cse.getCharArray(new char[256], 0);
			if( realType != CHAR_ARRAY_TYPE )
				throw new TestFailedException(this);
		}
		catch(ClassCastException e)
		{
			if( realType == CHAR_ARRAY_TYPE )
				throw new TestFailedException(this);
		}
		try
		{
			cse.getByteArray();
			if( realType != BYTE_ARRAY_TYPE )
				throw new TestFailedException(this);
		}
		catch(ClassCastException e)
		{
			if( realType == BYTE_ARRAY_TYPE )
				throw new TestFailedException(this);
		}
		try
		{
			cse.getByteArray(new byte[256], 0);
			if( realType != BYTE_ARRAY_TYPE )
				throw new TestFailedException(this);
		}
		catch(ClassCastException e)
		{
			if( realType == BYTE_ARRAY_TYPE )
				throw new TestFailedException(this);
		}
		try
		{
			cse.getInChannelHandle();
			if( realType != IN_CHANNEL_TYPE )
				throw new TestFailedException(this);
		}
		catch(ClassCastException e)
		{
			if( realType == IN_CHANNEL_TYPE )
				throw new TestFailedException(this);
		}
		try
		{
			cse.getAddress();
			if( realType != ADDRESS_TYPE )
				throw new TestFailedException(this);
		}
		catch(ClassCastException e)
		{
			if( realType == ADDRESS_TYPE )
				throw new TestFailedException(this);
		}
		try
		{
			cse.getBufferHandle();
			if( realType != BUFFER_HANDLE_TYPE )
				throw new TestFailedException(this);
		}
		catch(ClassCastException e)
		{
			if( realType == BUFFER_HANDLE_TYPE )
				throw new TestFailedException(this);
		}
		try
		{
			cse.getCommSpaceHandle();
			if( realType != COMM_SPACE_HANDLE_TYPE )
				throw new TestFailedException(this);
		}
		catch(ClassCastException e)
		{
			if( realType == COMM_SPACE_HANDLE_TYPE )
				throw new TestFailedException(this);
		}
		try
		{
			cse.getCommQueue();
			if( realType != COMM_QUEUE_TYPE )
				throw new TestFailedException(this);
		}
		catch(ClassCastException e)
		{
			if( realType == COMM_QUEUE_TYPE )
				throw new TestFailedException(this);
		}
		try
		{
			cse.getCommMonitor();
			if( realType != COMM_MONITOR_TYPE )
				throw new TestFailedException(this);
		}
		catch(ClassCastException e)
		{
			if( realType == COMM_MONITOR_TYPE )
				throw new TestFailedException(this);
		}
	}

	/**
	 * Compare an array of characters
	 */
	private boolean cmpCharArray(char a1[], char a2[])
	{
		boolean retval = true;

		if( a1.length != a2.length )
			retval = false;
		else
		{
			int lpc;

			for( lpc = 0; lpc < a1.length; lpc++ )
				retval = (a1[lpc] == a2[lpc]);
		}
		return( retval );
	}

	/**
	 * Compare an array of bytes
	 */
	private boolean cmpByteArray(byte a1[], byte a2[])
	{
		boolean retval = true;
		
		if( a1.length != a2.length )
			retval = false;
		else
		{
			int lpc;

			for( lpc = 0; lpc < a1.length; lpc++ )
				retval = (a1[lpc] == a2[lpc]);
		}
		return( retval );
	}

	public void testUnit() throws TestFailedException
	{
		final int arraySize = 16;
		CommSpaceElement cse;
		int lpc;
		
		int intVal = 1;
		char charArray[] =
		{ 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l' };
		byte byteArray[] =
		{ 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16 };
		InChannelHandle ich = null;
		Address addr = null;
		BufferHandle bh = null;
		CommSpaceHandle csh = null;
		CommQueue cq = null;
		CommMonitor cm = null;
		
		this.enterSection(ENV_NONE, "Constructors");
		
		/* make some elements */
		cse = new CommSpaceElement(intVal);
		this.verifyType(cse, INT_TYPE);
		cse = new CommSpaceElement(charArray);
		this.test(this.cmpCharArray(cse.getCharArray(), charArray));
		this.verifyType(cse, CHAR_ARRAY_TYPE);
		cse = new CommSpaceElement(byteArray);
		this.test(this.cmpByteArray(cse.getByteArray(), byteArray));
		this.verifyType(cse, BYTE_ARRAY_TYPE);
		/*
		 * silliness to make sure they blow up on null pointers...
		 * not really sure if this should be checked or not
		 */
		try
		{
			new CommSpaceElement(ich);
			throw new TestFailedException(this);
		}
		catch(NullPointerException e)
		{
		}
		try
		{
			new CommSpaceElement(addr);
			throw new TestFailedException(this);
		}
		catch(NullPointerException e)
		{
		}
		try
		{
			new CommSpaceElement(bh);
			throw new TestFailedException(this);
		}
		catch(NullPointerException e)
		{
		}
		try
		{
			new CommSpaceElement(csh);
			throw new TestFailedException(this);
		}
		catch(NullPointerException e)
		{
		}
		try
		{
			new CommSpaceElement(cq);
			throw new TestFailedException(this);
		}
		catch(NullPointerException e)
		{
		}
		try
		{
			new CommSpaceElement(cm);
			throw new TestFailedException(this);
		}
		catch(NullPointerException e)
		{
		}
		/* finish making valid elements */
		ich = new InChannelHandle();
		cse = new CommSpaceElement(ich);
		this.verifyType(cse, IN_CHANNEL_TYPE);
		bh = BufferHandle.create();
		bh.newBuffer(0,1);
		cse = new CommSpaceElement(bh);
		this.verifyType(cse, BUFFER_HANDLE_TYPE);
		csh = new CommSpaceHandle();
		cse = new CommSpaceElement(csh);
		this.verifyType(cse, COMM_SPACE_HANDLE_TYPE);
		cq = new CommQueue();
		cse = new CommSpaceElement(cq);
		this.verifyType(cse, COMM_QUEUE_TYPE);
		cm = new CommMonitor();
		cse = new CommSpaceElement(cm);
		this.verifyType(cse, COMM_MONITOR_TYPE);
		this.enterSection(ENV_NONE, "accessors");
		cse = new CommSpaceElement(intVal);
		this.test(cse.getInt() == intVal);

		/*
		 * a wall used to make sure that the get arrays don't blow
		 * anything away
		 */
		char charWall[] = { 'n', 'o', 'p', 'q', 'r', 's', 't', 'u' };
		char charDest[] = new char[charArray.length
					  + (charWall.length * 2)];
		int pos, count;

		/* Check that getCharArray works */
		cse = new CommSpaceElement(charArray);
		this.test(this.cmpCharArray(cse.getCharArray(), charArray));
		System.arraycopy(charWall, 0, charDest, 0, charWall.length);
		System.arraycopy(charWall, 0,
				 charDest, charWall.length + charArray.length,
				 charWall.length);
		cse.getCharArray(charDest, charWall.length);
		for( pos = 0, count = 0;
		     count < charWall.length;
		     pos++, count++ )
		{
			this.test(charDest[pos] == charWall[count]);
		}
		for( count = 0; count < charArray.length; pos++, count++ )
		{
			this.test(charDest[pos] == charArray[count]);
		}
		for( count = 0; count < charWall.length; pos++, count++ )
		{
			this.test(charDest[pos] == charWall[count]);
		}

		/* Same as above, except for getByteArray */
		byte byteWall[] =
		{ (byte)0xde, (byte)0xad, (byte)0xbe, (byte)0xef };
		byte byteDest[] = new byte[byteArray.length
					  + (byteWall.length * 2)];
		
		cse = new CommSpaceElement(byteArray);
		this.test(this.cmpByteArray(cse.getByteArray(), byteArray));
		System.arraycopy(byteWall, 0, byteDest, 0, byteWall.length);
		System.arraycopy(byteWall, 0,
				 byteDest, byteWall.length + byteArray.length,
				 byteWall.length);
		cse.getByteArray(byteDest, byteWall.length);
		for( pos = 0, count = 0;
		     count < byteWall.length;
		     pos++, count++ )
		{
			this.test(byteDest[pos] == byteWall[count]);
		}
		for( count = 0; count < byteArray.length; pos++, count++ )
		{
			this.test(byteDest[pos] == byteArray[count]);
		}
		for( count = 0; count < byteWall.length; pos++, count++ )
		{
			this.test(byteDest[pos] == byteWall[count]);
		}
	}
}
