#!/usr/local/bin/perl -w
#
# javasel.pl
#
# Copyright (c) 1999, 2000 University of Utah and the Flux Group.
# All rights reserved.
#
# This file is part of Janos.
# 
# Janos is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License, version 2, as
# published by the Free Software Foundation.
# 
# Janos is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details, in the file
# COPYING.  If that file is not present, write to the Free
# Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
# 02111-1307, USA
# 
# The University of Utah and the Flux Group requests users to
# return to flux-dist@cs.utah.edu any improvements that they make.
#

#
# Select a java implementation, library, etc. by
# emitting the correct shell commands to set it up.
# 
# Depends on the existence of a JDKROOT directory formatted
# in the appropriate way.
#
# Usage: See below (in usage())
#

## Internal debugging of this script.
$dbg = 0;

## Make sure the environment variables this script depends on are defined
&checkEnv();

## Figure out where all the JDKS are linked from
$JANOSDIR = $ENV{'JANOSDIR'};
($JDKROOT = $ENV{'JDKROOT'}) 
  || ($JDKROOT = "$JANOSDIR" . '/jdks'); 

$VMvendor   = 'kaffe';  # 'kaffe' or 'sun'
$libVendor  = $VMvendor; # hold-over.  
$libVersion = '1.1';    # '1.0', '1.1' or '1.2'
$VMmethod   = '-jit';   # '-jit' or '-intrp'
$VMstyle    = '-debug'; # '-debug' or '-opt' or '-stat' or '-gprof' or '-xprof'
$customSuffix = '';
$arch 	    = 'unknown-unknown';
$shSyntax   = '-csh';

## Figure out what architecture we're running on
$arch        = &archName();

# Parse arguments
&parseArgs();

## Use either the -optimized or -debug libraries
if (($VMstyle eq '-opt') || ($VMstyle eq '-gprof') || ($VMstyle eq '-stat') || ($VMstyle eq '-xprof')) {
    $libStyle = 'optimized';
} else {
    $libStyle = 'debug';
}

## For each package (zip file) use the appropriate version for optimized/debugging

## look for impossible combinations of VM/library/version
&checkImpossible();

## Create the directory name.  Sun names are constructed one way,
## Kaffe/Alta names are constructed in another.
if ("${VMvendor}" eq 'sun') {
    $jvmDirectory = "${JDKROOT}/${arch}/${VMvendor}-${libVersion}${customSuffix}";
} else {
    $jvmDirectory = "${JDKROOT}/${arch}/${VMvendor}-${libVersion}-${libVendor}${VMmethod}${VMstyle}${customSuffix}";
}  

## Make sure the request combination actually exists
if (! -e "${jvmDirectory}") {
    print STDERR "FATAL: The directory $jvmDirectory does not exist.\n";
    exit 11;
}

if (! -e "${jvmDirectory}/bin") {
    print STDERR "FATAL: The directory $jvmDirectory/bin does not exist.\n";
    exit 11;
}

## Pull old JDKROOT entries out of PATH
foreach $entry (split(/:/, $ENV{'PATH'})) {
    if ($entry =~ /^$JDKROOT/) {
	&dbgPrint ("Dropping $entry from PATH\n");
	# skip it
    } elsif ($entry ne ".") {
	push(@NEWPATH, $entry);
    }
}

$newPath = join(':', @NEWPATH);
&dbgPrint("New PATH: $newPath\n");

## Pull old JDKROOT entries out of LD_LIBRARY_PATH
if (defined $ENV{'LD_LIBRARY_PATH'}) {
    foreach $entry (split(/:/, $ENV{'LD_LIBRARY_PATH'})) {
	if ($entry =~ /^$JDKROOT/) {
	    &dbgPrint ("Dropping $entry from LD_LIBRARY_PATH\n");
	    # skip it
	} else {
	    push(@NEWLDPATH, $entry);
	}
    }
    
    $newLdPath = join(':', @NEWLDPATH);
    &dbgPrint("New LD_LIBRARY_PATH: $newLdPath\n");
}

# Open the tmp shell script to write to
open (SHELLSCRIPT, ">&STDOUT")
  || die "Cannot dup STDOUT for writing.";

# All platforms, kill some vars:
&unsetEnv('JAVA_HOME');
&unsetEnv('KAFFEHOME');

##
## Architecture and VM-specific environment variables
##

if (($arch =~ /^AIX/) && ($VMvendor eq 'sun')) {
    # Tell it to JIT (only used by JDK 1.1.2+)
    &setEnv('JAVA_COMPILER','jitc');
    
} elsif ($arch =~ /^SunOS/) {
    # Tack /usr/bin on the front if its not already there.
    $newPath = "/usr/bin:" . $newPath
      if (index($newPath, "/usr/bin") != $[);
    
} elsif ($VMvendor eq "kaffe") {
    &setEnv('KAFFEHOME', $jvmDirectory);
    if ($newLdPath) {
	&setEnv('LD_LIBRARY_PATH', "${newLdPath}:" . $jvmDirectory . "/lib");
    } else {
	&setEnv('LD_LIBRARY_PATH', $jvmDirectory . "/lib");
    }
    &setEnv('KAFFELIBRARYPATH', $jvmDirectory . "/share/kaffe/lib/i386-freebsd2");
}


## Define the executable to launch the JVM
$java = 'kaffe'; 
$java = 'java' if $VMvendor eq 'sun';
# XXX $java = 'alta' if $VMvendor eq 'alta';

# All platforms set the basic vars:

# PATH
&setEnv('PATH',      "${newPath}:" . $jvmDirectory . "/bin:.");

# JAVA_HOME
&setEnv('JAVA_HOME', $jvmDirectory);

# CLASSPATH
if (($VMvendor eq 'sun') && ($libVersion ne '1.2')) {
    $classpath = '.:' . $jvmDirectory . "/lib/classes.zip";
} else {
    $classpath = '.';
}
&dbgPrint("CLASSPATH= $classpath\n");
&checkPathElements($classpath);
&setEnv('CLASSPATH', $classpath);

# JAVAEXE (the name of the executable)
&setEnv('JAVAEXE', $java);

# Define the JAVA_OPT environment variable so that the kaffeh-em scripts
# know which version of the header files they are generating.
if (($VMstyle eq '-opt') || ($VMstyle eq '-gprof') || ($VMstyle eq '-xprof') || ($VMstyle eq '-stat')) {
    &setEnv('JAVA_OPT', '-optimized');
} else {
    &setEnv('JAVA_OPT', $VMstyle);  # -debug
}

$testProg  = 'test'; # = '/bin/test';

## Send the commands to define the environment
if ($shSyntax eq '-sh') {
  $echoEnv = "$testProg \"\$CLASSPATH\"x != x && echo CLASSPATH \$CLASSPATH; \
		$testProg \"\$LD_LIBRARY_PATH\"x != x && echo LD_LIBRARY_PATH \$LD_LIBRARY_PATH; \
		$testProg \"\$JAVA_HOME\"x != x && echo JAVA_HOME \$JAVA_HOME; \
		$testProg x != \"\$KAFFEHOME\"x && echo KAFFEHOME \$KAFFEHOME; \
		ls -la `which $java`";
} else {
  $echoEnv = "$testProg \$?CLASSPATH -ne 0 && echo CLASSPATH \$CLASSPATH; \
		$testProg \$?LD_LIBRARY_PATH -ne 0 && echo LD_LIBRARY_PATH \$LD_LIBRARY_PATH; \
		$testProg \$?JAVA_HOME -ne 0 && echo JAVA_HOME \$JAVA_HOME; \
		$testProg \$?KAFFEHOME -ne 0 && echo KAFFEHOME \$KAFFEHOME; \
		ls -la `which $java`";
}
print SHELLSCRIPT "$echoEnv;\n";

## Define the 'jver' alias to print info about the current configuration
if ("$VMvendor" ne 'sun') {
  $javaselConfig = "echo \"VM=${VMvendor}; lib=${libVendor}-${libVersion}; VMmethod=${VMmethod}; VMstyle=${VMstyle}\"";
} else {
  $javaselConfig = "echo \"VM=${VMvendor}; lib=${libVendor}-${libVersion}\"";
}
$javaVer = "$java -version";

if ($shSyntax eq '-csh') {
  print SHELLSCRIPT "alias jver \'$javaselConfig; $echoEnv; (unsetenv KAFFE_DEBUG; $javaVer)\';\n";
} else {
  print SHELLSCRIPT "jver() { $javaselConfig; $echoEnv; KAFFE_DEBUG= $javaVer; } ;\n";
}

# ------------------------------

sub dieImpossible {
    print STDERR "Impossible: @_\n";
    exit 11;
}

sub checkImpossible {
    &usage ("ERROR: VM must be either 'alta', 'kaffe' or 'sun'")
	if (("$VMvendor" ne 'kaffe') && ("$VMvendor" ne 'sun') && ("$VMvendor" ne 'alta'));

    &usage ("ERROR: Lib Vendor must be either 'sun', or 'kaffe'")
	if (("$libVendor" ne 'sun') && ("$libVendor" ne 'kaffe'));

    &usage ("ERROR: Lib Version must be either '1.0', '1.1' or '1.2'")
	if (("$libVersion" ne '1.0') && ("$libVersion" ne '1.1') && ("$libVersion" ne '1.2'));

    &dieImpossible ("Kaffe library 1.0 doesn't exist.")
	if "${libVendor}-${libVersion}" eq "kaffe-1.0";
    
    &dieImpossible ("VM and Library vendor must match.")
	if ("${VMvendor}" ne "${libVendor}");
    
    &dieImpossible ("Sun doesn't support Kaffe's libraries") 
	if (("${VMvendor}" eq 'sun') && ("${libVendor}" ne 'sun'));
    
    &dieImpossible ("Kaffe 1.0 doesn't exist anywhere.")
	if ("${VMvendor}-${libVersion}" eq "kaffe-1.0");
}

sub setEnv {
    local($var, $value) = @_;
    
    if ($shSyntax eq '-csh') {
      print SHELLSCRIPT "setenv $var \"$value\";\n";
    } else {
      print SHELLSCRIPT "export $var; $var=\"$value\";\n";
    }
}

sub unsetEnv {
    local($var) = @_;
    
    if ($shSyntax eq '-csh') {
      print SHELLSCRIPT "unsetenv $var;\n";
    } else {
      print SHELLSCRIPT "$var=;\n";
    }
}

## Name of arch we're running on--
##  'solaris', 'bsd', 'linux', 'aix'
sub archName {
    local ($uname);
    open (UNAME, "uname -s -m|") || die 'FATAL: Cannot "uname -s -m" on this machine';
    $uname = <UNAME>;
    $uname =~ s/ /-/g;

    &dbgPrint("UNAME: $uname\n");

    chop $uname;
    return $uname;
}


sub parseArgs {
    while ($_ = $ARGV[0]) {
	last if /^--$/;  # allow "--" to stop option arg parsing
	last if /^[^-]/; # end on first non-switch

	shift @ARGV; # eat the option

	&dbgPrint ("parseArgs(): Start with: $#ARGV $_\n");

	## Options for selecting the VM bytecode eval method
	/^-intrp$/ && do {($VMmethod = '-intrp'); next;};
	/^-jit$/ && do {($VMmethod = '-jit'); next;};

	## Options for selecting the VM style (debugging, optimized or profiling)
	/^-opt$/ && do {($VMstyle = '-opt'); next;};
	/^-gprof$/ && do {($VMstyle = '-gprof'); next;};
	/^-xprof$/ && do {($VMstyle = '-xprof'); next;};
	/^-stat$/ && do {($VMstyle = '-stat'); next;};
	/^-debug$/ && do {($VMstyle = '-debug'); next;};

	/^-sh$/ && do {($shSyntax = '-sh'); next;};
	/^-csh$/ && do {($shSyntax = '-csh'); next;};

	## Customized directory suffix
	/^--([^ ]*)/ && do {($customSuffix = "-$1"); next;};

	## -h for usage
	/^-h$/ && do {(&usage("Help:")); next;};
	## -d for debugging
	/^-d$/ && do {($dbg = 1); next;};
	## -o <file> provides the name of the shell script file
	#if (/^-o$/) {
	#    ($#ARGV >= 0) || &usage("-o must be given a filename");
	#    $tmpShellScript = shift @ARGV;
	#    next;
	#}
	## Unknown -args are fatal
	/^\-/ && (&usage("Unknown option: $_"));
    }

    ## At this point remaing args are VMvendor, libVendor, and libVersion
    &dbgPrint ("parseArgs(): Post switch parse: $#ARGV, @ARGV\n");
    $ARGV[0] && ($VMvendor = shift @ARGV);
    $libVendor = 'sun' if ($VMvendor eq 'sun'); 
    &dbgPrint ("parseArgs(): VMvendor is $VMvendor\n");
    $ARGV[0] && ($libVendor = shift @ARGV);
    &dbgPrint ("parseArgs(): libVendor is $libVendor\n");
    $ARGV[0] && ($libVersion = shift @ARGV);
    &dbgPrint ("parseArgs(): libVersion is $libVersion\n");
} 

sub checkEnv {
  $failure = 0;

  if (! $ENV{'JANOSDIR'}) {
    print STDERR "JANOSDIR environment variable is not defined.  This script depends on it.";
    $failure = 1;
  }

  exit 2 if ($failure);
}

sub usage {
    print STDERR "@_\n";
    print STDERR "javasel.pl <optarg> [<VM Vendor> [<lib Vendor> [<lib Version]]]\n";
    print STDERR "\t<optarg>:\t'-sh'|'-csh', '-opt'|'-debug'|'-stat'|'-gprof'|'-xprof', '--magic', and/or '-jit'|'-intrp'\n";
    print STDERR "\t<VM Vendor>:\t'kaffe' or 'sun'; default $VMvendor\n";
    print STDERR "\t<lib Vendor>:\t'kaffe' or 'sun'; currently $libVendor\n";
    print STDERR "\t<lib Version>:\t'1.0', '1.1', or '1.2'; currently $libVersion\n";
    print STDERR "\n";
    exit 1;
}

sub dbgPrint {
    if ($dbg) {
	print STDERR ("@_");
    }
}

sub checkPathElements {
    local(@paths) = split(/:/,shift);
    
    foreach $path (@paths) {
	next if $path eq '.';
	-s $path || die ("ERROR: Required CLASSPATH element \"$path\" does not exist.\n");
    }
}
