# utf8munge.pl
#
# Copyright (c) 1999, 2000 University of Utah and the Flux Group.
# All rights reserved.
#
# This file is part of Janos.
# 
# Janos is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License, version 2, as
# published by the Free Software Foundation.
# 
# Janos is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details, in the file
# COPYING.  If that file is not present, write to the Free
# Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
# 02111-1307, USA
# 
# The University of Utah and the Flux Group requests users to
# return to flux-dist@cs.utah.edu any improvements that they make.
# 

#
# Munge the Utf8 constants in a .class file.
#
# Usage: utf8munge.pl [option] <old.class> <new.class> <old str> <new str> [<oldstr> <newstr> ...]
#
# Options:
#	-changeClassName: change the name of the actual class object.
#

use JavaClass;

$changeClassName = 0;

## Parse the required command line
my $classFile = shift || &usage();
if ($classFile eq '-changeClassName') {
  $changeClassName = 1;
  $classFile = shift || &usage();
}
my $newClass  = shift || &usage();
my @oldStr;
my @newStr;

$oldStr[0] = shift || &usage();
$newStr[0] = shift || &usage();

if (!(defined($newClass) && defined($oldStr[0]) && defined($newStr[0]))) {
    print STDOUT ("Usage: utf8munge.pl [option] <old.class> <new.class> <old str> <new str> [<oldstr> <newstr> ...]\n");
    exit 11;
}

## Now parse any additional str mucking parameters
my $i = 1;
while ($#ARGV >= 0) {
  $oldStr[$i] = shift;
  #die "$oldStr[$i] must be followed by a replacement string" if (!$#ARGV);
  $newStr[$i] = shift;
  $i++;
}

## for printing:
#$JavaClass::detailedFields = 0;
#$JavaClass::detailedMethods = 0;

my $class = &JavaClass::readClass($classFile);


if (!$changeClassName) {
  ## We don't want to change the "name" of this class, so to be safe, we
  ## just create an additional Class entry and an additional Utf8 entry
  ## which contain the old info This is only a problem if any of the
  ## $oldStr match 'thisClass->name'.
  $thisClassEntry = $class->{constantPool}[$class->{thisClass}];
  $ni = $thisClassEntry->{nameIndex};
  my $name = $class->{constantPool}[$ni]->{val};
  
  if (grep($name =~ m/$_/, @oldStr)) {
    # Save copies of the thisClass entry and the Utf8String it
    # points to.  These copies won't get munged in the frenzy below.
    
    %classEntry = %{$thisClassEntry};
    %origUtf8   = %{$class->{constantPool}[$ni]};
  } 
}


### transform class
foreach $i (1..(($class->{constantPoolCt})-1)) {
    $cpEntry = $class->{constantPool}[$i];

    ### Change the Utf8Strings
    if ($cpEntry->{tag} eq $JavaClass::CONSTANT_Utf8) {
        $val = $cpEntry->{val};

	## Apply each transform (in order!)
	foreach $s (0..$#oldStr) {
	  $val =~ s,$oldStr[$s],$newStr[$s],g;
        }

        # print "OLD: $cpEntry->{val}; NEW: $val\n";

        $cpEntry->{val} = $val;
        $cpEntry->{len} = length($val);
    }
}

# Insert the saved entries, if they were defined.
if ((!$changeClassName) && defined(%classEntry)) {
    $ct = $class->{constantPoolCt};

    # Insert the saved Utf8 entry at the end of the constant pool
    $class->{constantPool}[$ct] = \%origUtf8;
    
    # Patch up the saved classEntry to point to this utf8
    $classEntry{nameIndex} = $ct;

    $ct++;

    # Insert the classEntry in the constant pool
    $class->{constantPool}[$ct] = \%classEntry;

    # Patch the thisClass pointer
    $class->{thisClass} = $ct;

    $ct++;

    # Patch the constantPoolCt
    $class->{constantPoolCt} = $ct;
}

###
#&JavaClass::printClass($class);
&JavaClass::writeClass($class, "$newClass");

####
####
####

sub usage() {
  print STDOUT "Usage:\n";
  print STDOUT "\tutf8munge.pl [option] <old.class> <new.class> <old str> <new str> [<oldstr> <newstr> ...]\n";
  print STDOUT "\t\tThe only [option] is -changeClassName.  By default the name will not be changed.\n";

  exit 11;
}

# eof
